#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_vpn_community_star
short_description: Manages vpn-community-star objects on Check Point over Web Services API
description:
  - Manages vpn-community-star objects on Check Point devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "1.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  center_gateways:
    description:
      - Collection of center VPN Gateway and VPN Device objects identified by the name or UID.
    type: list
    elements: str
  disable_nat:
    description:
      - Indicates whether to disable NAT inside the VPN Community.
      - Available from R82 JHF management version.
    type: bool
    version_added: "6.5.0"
  disable_nat_on:
    description:
      - Indicates on which gateways to disable NAT inside the VPN Community.
      - Available from R82 JHF management version.
    type: str
    choices: ['satellite gateways only', 'both center and satellite gateways']
    version_added: "6.5.0"
  encrypted_traffic:
    description:
      - Encrypted traffic settings.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      enabled:
        description:
          - Indicates whether to accept all encrypted traffic.
        type: bool
      community_members:
        description:
          - Indicates on which community members to accept all encrypted traffic.
        type: str
        choices: ['satellite gateways only', 'both center and satellite gateways']
  encryption_method:
    description:
      - The encryption method to be used.
      - Available from R80.10 management version.
    type: str
    choices: ['prefer ikev2 but support ikev1', 'ikev2 only', 'ikev1 for ipv4 and ikev2 for ipv6 only']
  encryption_suite:
    description:
      - The encryption suite to be used.
      - Available from R80.10 management version.
    type: str
    choices: ['suite-b-gcm-256', 'custom', 'vpn b', 'vpn a', 'suite-b-gcm-128']
  excluded_services:
    description:
      - Collection of services that are excluded from the community identified by the name or UID.<br> Connections with these services will not be
        encrypted and will not match rules specifying the community in the VPN community.
      - Available from R82 JHF management version.
    type: list
    elements: str
    version_added: "6.5.0"
  granular_encryptions:
    description:
      - VPN granular encryption settings.
      - Available from R81 management version.
    type: list
    elements: dict
    version_added: "5.1.0"
    suboptions:
      internal_gateway:
        description:
          - Internally managed Check Point gateway identified by name or UID, or 'Any' for all internal-gateways participants in this community.
        type: str
      external_gateway:
        description:
          - Externally managed or 3rd party gateway identified by name or UID.
        type: str
      encryption_method:
        description:
          - The encryption method to be used.
        type: str
        choices: ['prefer ikev2 but support ikev1', 'ikev2 only', 'ikev1 for ipv4 and ikev2 for ipv6 only']
      encryption_suite:
        description:
          - The encryption suite to be used.
        type: str
        choices: ['suite-b-gcm-256', 'custom', 'vpn b', 'vpn a', 'suite-b-gcm-128']
      ike_phase_1:
        description:
          - Ike Phase 1 settings. Only applicable when the encryption-suite is set to [custom].
        type: dict
        suboptions:
          encryption_algorithm:
            description:
              - The encryption algorithm to be used.
            type: str
            choices: ['cast', 'aes-256', 'des', 'aes-128', '3des']
          data_integrity:
            description:
              - The hash algorithm to be used.
            type: str
            choices: ['aes-xcbc', 'sha1', 'sha256', 'sha384', 'sha512', 'md5']
          diffie_hellman_group:
            description:
              - The Diffie-Hellman group to be used.
            type: str
            choices: ['group-1', 'group-2', 'group-5', 'group-14', 'group-15', 'group-16', 'group-17', 'group-18', 'group-19', 'group-20', 'group-24']
          ike_p1_rekey_time:
            description:
              - Indicates the time interval for IKE phase 1 renegotiation.
              - Available from R81 management version.
            type: int
          ike_p1_rekey_time_unit:
            description:
              - Indicates the time unit for [ike-p1-rekey-time-unit] parameter, rounded up to minutes scale.
              - Available from R81 management version.
            type: str
            choices: ['days', 'hours', 'minutes', 'seconds']
      ike_phase_2:
        description:
          - Ike Phase 2 settings. Only applicable when the encryption-suite is set to [custom].
        type: dict
        suboptions:
          encryption_algorithm:
            description:
              - The encryption algorithm to be used.
            type: str
            choices: ['cast', 'aes-gcm-256', 'cast-40', 'aes-256', 'des', 'aes-128', '3des', 'des-40cp', 'aes-gcm-128', 'none']
          data_integrity:
            description:
              - The hash algorithm to be used.
            type: str
            choices: ['aes-xcbc', 'sha1', 'sha256', 'sha384', 'sha512', 'md5']
          ike_p2_use_pfs:
            description:
              - Indicates whether Perfect Forward Secrecy (PFS) is being used for IKE phase 2.
            type: bool
          ike_p2_pfs_dh_grp:
            description:
              - The Diffie-Hellman group to be used.
            type: str
            choices: ['group-1', 'group-2', 'group-5', 'group-14', 'group-15', 'group-16', 'group-17', 'group-18', 'group-19', 'group-20', 'group-24']
          ike_p2_rekey_time:
            description:
              - Indicates the time interval for IKE phase 2 renegotiation.
              - Available from R81 management version.
            type: int
          ike_p2_rekey_time_unit:
            description:
              - Indicates the time unit for [ike-p2-rekey-time-unit] parameter.
              - Available from R81 management version.
            type: str
            choices: ['days', 'hours', 'minutes', 'seconds']
  ike_phase_1:
    description:
      - Ike Phase 1 settings. Only applicable when the encryption-suite is set to [custom].
      - Available from R80.10 management version.
    type: dict
    suboptions:
      data_integrity:
        description:
          - The hash algorithm to be used.
        type: str
        choices: ['aes-xcbc', 'sha1', 'sha256', 'sha384', 'sha512', 'md5']
      diffie_hellman_group:
        description:
          - The Diffie-Hellman group to be used.
        type: str
        choices: ['group-1', 'group-2', 'group-5', 'group-14', 'group-19', 'group-20']
      encryption_algorithm:
        description:
          - The encryption algorithm to be used.
        type: str
        choices: ['cast', 'aes-256', 'des', 'aes-128', '3des']
      ike_p1_rekey_time:
        description:
          - Indicates the time interval for IKE phase 1 renegotiation.
          - Available from R81 management version.
        type: int
        version_added: "5.1.0"
      ike_p1_rekey_time_unit:
        description:
          - Indicates the time unit for [ike-p1-rekey-time-unit] parameter, rounded up to minutes scale.
          - Available from R81 management version.
        type: str
        choices: ['days', 'hours', 'minutes', 'seconds']
        version_added: "5.1.0"
  ike_phase_2:
    description:
      - Ike Phase 2 settings. Only applicable when the encryption-suite is set to [custom].
      - Available from R80.10 management version.
    type: dict
    suboptions:
      data_integrity:
        description:
          - The hash algorithm to be used.
        type: str
        choices: ['aes-xcbc', 'sha1', 'sha256', 'sha384', 'sha512', 'md5']
      encryption_algorithm:
        description:
          - The encryption algorithm to be used.
        type: str
        choices: ['cast', 'aes-gcm-256', 'cast-40', 'aes-256', 'des', 'aes-128', '3des', 'des-40cp', 'aes-gcm-128', 'none']
      ike_p2_use_pfs:
        description:
          - Indicates whether Perfect Forward Secrecy (PFS) is being used for IKE phase 2.
          - Available from R81 management version.
        type: bool
        version_added: "5.1.0"
      ike_p2_pfs_dh_grp:
        description:
          - The Diffie-Hellman group to be used.
        type: str
        choices: ['group-1', 'group-2', 'group-5', 'group-14', 'group-15', 'group-16', 'group-17', 'group-18', 'group-19', 'group-20', 'group-24']
        version_added: "5.1.0"
      ike_p2_rekey_time:
        description:
          - Indicates the time interval for IKE phase 2 renegotiation.
          - Available from R81 management version.
        type: int
        version_added: "5.1.0"
      ike_p2_rekey_time_unit:
        description:
          - Indicates the time unit for [ike-p2-rekey-time-unit] parameter.
          - Available from R81 management version.
        type: str
        choices: ['days', 'hours', 'minutes', 'seconds']
        version_added: "5.1.0"
  mep:
    description:
      - Multiple Entry Point properties.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      enabled:
        description:
          - Enable center gateways as Multiple Entry Points.
        type: bool
      entry_point_selection_mechanism:
        description:
          - The method by which the entry point gateway will be chosen from the gateways in the center.
        type: str
        choices: ['closest gateway to source', 'closest gateway to destination', 'random selection', 'manual']
      entry_point_final_selection_mechanism:
        description:
          - The method by which the final entry point gateway will be chosen when the chosen mechanism returns more than one optional entry point.
        type: str
        choices: ['random selection', 'first to respond']
      tracking:
        description:
          - Tracking option for the MEP.
        type: str
        choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                 'user defined alert no.3']
      default_priority_rule:
        description:
          - Priority rule for all satellite gateways. Relevant only if 'entry-point-selection-mechanism' is set to 'manual'.
        type: dict
        suboptions:
          first_priority_center_gateways:
            description:
              - Collection of first priority center gateways identified by the name or UID.
            type: list
            elements: str
          second_priority_center_gateways:
            description:
              - Collection of second priority center gateways identified by the name or UID.
            type: list
            elements: str
          third_priority_center_gateways:
            description:
              - Collection of third priority center gateways identified by the name or UID.
            type: list
            elements: str
      exception_priority_rules:
        description:
          - Exception priority rules for specific satellites gateways. Relevant only if 'entry-point-selection-mechanism' is set to 'manual'.
        type: list
        elements: dict
        suboptions:
          satellite_gateways:
            description:
              - Collection of satellite gateways to apply priority rules on identified by the name or UID.
            type: list
            elements: str
          first_priority_center_gateways:
            description:
              - Collection of first priority center gateways identified by the name or UID.
            type: list
            elements: str
          second_priority_center_gateways:
            description:
              - Collection of second priority center gateways identified by the name or UID.
            type: list
            elements: str
          third_priority_center_gateways:
            description:
              - Collection of third priority center gateways identified by the name or UID.
            type: list
            elements: str
  mesh_center_gateways:
    description:
      - Indicates whether the meshed community is in center.
    type: bool
  override_vpn_domains:
    description:
      - The Overrides VPN Domains of the participants GWs.
      - Available from R80.40 management version.
    type: list
    elements: dict
    version_added: "5.1.0"
    suboptions:
      gateway:
        description:
          - Participant gateway in override VPN domain identified by the name or UID.
        type: str
      vpn_domain:
        description:
          - VPN domain network identified by the name or UID.
        type: str
  permanent_tunnels:
    description:
      - Permanent tunnels properties.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      set_permanent_tunnels:
        description:
          - Indicates which tunnels to set as permanent.
        type: str
        choices: ['on all tunnels in the community', 'on all tunnels of specific gateways', 'on specific tunnels in the community', 'off']
      gateways:
        description:
          - List of gateways to set all their tunnels to permanent with specified track options. Will take effect only if set-permanent-tunnels-on
            is set to all-tunnels-of-specific-gateways.
        type: list
        elements: dict
        suboptions:
          gateway:
            description:
              - Gateway to set all is tunnels to permanent with specified track options.<br> Identified by name or UID.
            type: str
          track_options:
            description:
              - Indicates whether to use the community track options or to override track options for the permanent tunnels.
            type: str
            choices: ['according to community track options', 'override track options']
          override_tunnel_down_track:
            description:
              - Gateway tunnel down track option. Relevant only if the track-options is set to 'override track options'.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
          override_tunnel_up_track:
            description:
              - Gateway tunnel up track option. Relevant only if the track-options is set to 'override track options'.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
      tunnels:
        description:
          - List of tunnels to set as permanent with specified track options. Will take effect only if set-permanent-tunnels-on is set to
            specific-tunnels-in-the-community.
        type: list
        elements: dict
        suboptions:
          first_tunnel_endpoint:
            description:
              - First tunnel endpoint (center gateway). Identified by name or UID.
            type: str
          second_tunnel_endpoint:
            description:
              - Second tunnel endpoint (center gateway for meshed VPN community and satellite gateway for star VPN community). Identified by name or UID.
            type: str
          track_options:
            description:
              - Indicates whether to use the community track options or to override track options for the permanent tunnels.
            type: str
            choices: ['according to community track options', 'override track options']
          override_tunnel_down_track:
            description:
              - Gateway tunnel down track option. Relevant only if the track-options is set to 'override track options'.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
          override_tunnel_up_track:
            description:
              - Gateway tunnel up track option. Relevant only if the track-options is set to 'override track options'.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
      rim:
        description:
          - Route Injection Mechanism settings.
        type: dict
        suboptions:
          enabled:
            description:
              - Indicates whether Route Injection Mechanism is enabled.
            type: bool
          enable_on_center_gateways:
            description:
              - Indicates whether to enable automatic Route Injection Mechanism on center gateways.
            type: bool
          enable_on_satellite_gateways:
            description:
              - Indicates whether to enable automatic Route Injection Mechanism on satellite gateways.
            type: bool
          route_injection_track:
            description:
              - Route injection track method.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
      tunnel_down_track:
        description:
          - VPN community permanent tunnels down track option.
        type: str
        choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                 'user defined alert no.3']
      tunnel_up_track:
        description:
          - Permanent tunnels up track option.
        type: str
        choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                 'user defined alert no.3']
  satellite_gateways:
    description:
      - Collection of Gateway objects representing satellite gateways identified by the name or UID.
    type: list
    elements: str
  shared_secrets:
    description:
      - Shared secrets for external gateways.
      - Available from R80.10 management version.
    type: list
    elements: dict
    suboptions:
      external_gateway:
        description:
          - External gateway identified by the name or UID.
        type: str
      shared_secret:
        description:
          - Shared secret.
        type: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  tunnel_granularity:
    description:
      - VPN tunnel sharing option to be used.
      - Available from R81 management version.
    type: str
    choices: ['per_host', 'per_subnet', 'universal']
    version_added: "5.1.0"
  use_shared_secret:
    description:
      - Indicates whether the shared secret should be used for all external gateways.
      - Available from R80.10 management version.
    type: bool
  vpn_routing:
    description:
      - Enable VPN routing to satellites.
      - Available from R82 JHF management version.
    type: str
    choices: ['to center only', 'to center and to other satellites', 'to center other satellites and internet']
    version_added: "6.5.0"
  wire_mode:
    description:
      - VPN Community Wire mode properties.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      allow_uninspected_encrypted_traffic:
        description:
          - Allow uninspected encrypted traffic between Wire mode interfaces of this Community members.
        type: bool
      allow_uninspected_encrypted_routing:
        description:
          - Allow members to route uninspected encrypted traffic in VPN routing configurations.
        type: bool
  routing_mode:
    description:
      - VPN Community Routing Mode.
      - Available from R82 JHF management version.
    type: str
    choices: ['domain_based', 'route_based']
    version_added: "6.5.0"
  advanced_properties:
    description:
      - Advanced properties.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      support_ip_compression:
        description:
          - Indicates whether to support IP compression.
        type: bool
      use_aggressive_mode:
        description:
          - Indicates whether to use aggressive mode.
        type: bool
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-vpn-community-star
  cp_mgmt_vpn_community_star:
    center_gateways: Second_Security_Gateway
    encryption_method: prefer ikev2 but support ikev1
    encryption_suite: custom
    ike_phase_1:
      data_integrity: sha1
      diffie_hellman_group: group 19
      encryption_algorithm: aes-128
    ike_phase_2:
      data_integrity: aes-xcbc
      encryption_algorithm: aes-gcm-128
    name: New_VPN_Community_Star_1
    state: present

- name: set-vpn-community-star
  cp_mgmt_vpn_community_star:
    encryption_method: ikev2 only
    encryption_suite: custom
    ike_phase_1:
      data_integrity: sha1
      diffie_hellman_group: group 19
      encryption_algorithm: aes-128
    ike_phase_2:
      data_integrity: aes-xcbc
      encryption_algorithm: aes-gcm-128
    name: New_VPN_Community_Star_1
    state: present

- name: delete-vpn-community-star
  cp_mgmt_vpn_community_star:
    name: New_VPN_Community_Star_1
    state: absent
"""

RETURN = """
cp_mgmt_vpn_community_star:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        center_gateways=dict(type="list", elements="str"),
        disable_nat=dict(
            type="bool"
        ),

        disable_nat_on=dict(
            type="str",
            choices=[
                "satellite gateways only",
                "both center and satellite gateways"
            ]
        ),

        encrypted_traffic=dict(
            type="dict",
            options=dict(
                enabled=dict(type="bool", ),
                community_members=dict(
                    type="str",
                    choices=[
                        "satellite gateways only",
                        "both center and satellite gateways"
                    ]
                )
            )
        ),
        encryption_method=dict(
            type="str",
            choices=[
                "prefer ikev2 but support ikev1",
                "ikev2 only",
                "ikev1 for ipv4 and ikev2 for ipv6 only",
            ],
        ),
        encryption_suite=dict(
            type="str",
            choices=[
                "suite-b-gcm-256",
                "custom",
                "vpn b",
                "vpn a",
                "suite-b-gcm-128",
            ],
        ),
        excluded_services=dict(type="list", elements="str"),
        granular_encryptions=dict(type='list', elements="dict", options=dict(
            internal_gateway=dict(type='str'),
            external_gateway=dict(type='str'),
            encryption_method=dict(type='str', choices=['prefer ikev2 but support ikev1', 'ikev2 only',
                                                        'ikev1 for ipv4 and ikev2 for ipv6 only']),
            encryption_suite=dict(type='str',
                                  choices=['suite-b-gcm-256', 'custom', 'vpn b', 'vpn a', 'suite-b-gcm-128']),
            ike_phase_1=dict(type='dict', options=dict(
                encryption_algorithm=dict(type='str', choices=['cast', 'aes-256', 'des', 'aes-128', '3des']),
                data_integrity=dict(type='str', choices=['aes-xcbc', 'sha1', 'sha256', 'sha384', 'sha512', 'md5']),
                diffie_hellman_group=dict(type='str', choices=['group-1', 'group-2', 'group-5',
                                                               'group-14', 'group-15', 'group-16', 'group-17',
                                                               'group-18', 'group-19', 'group-20',
                                                               'group-24']),
                ike_p1_rekey_time=dict(type='int', no_log=False),
                ike_p1_rekey_time_unit=dict(type='str', choices=['days', 'hours', 'minutes', 'seconds'])
            )),
            ike_phase_2=dict(type='dict', options=dict(
                encryption_algorithm=dict(type='str', choices=['cast', 'aes-gcm-256', 'cast-40',
                                                               'aes-256', 'des', 'aes-128', '3des', 'des-40cp',
                                                               'aes-gcm-128', 'none']),
                data_integrity=dict(type='str', choices=['aes-xcbc', 'sha1', 'sha256', 'sha384', 'sha512', 'md5']),
                ike_p2_use_pfs=dict(type='bool'),
                ike_p2_pfs_dh_grp=dict(type='str', choices=['group-1', 'group-2', 'group-5',
                                                            'group-14', 'group-15', 'group-16', 'group-17', 'group-18',
                                                            'group-19', 'group-20', 'group-24']),
                ike_p2_rekey_time=dict(type='int', no_log=False),
                ike_p2_rekey_time_unit=dict(type='str', choices=['days', 'hours', 'minutes', 'seconds']),
            ))
        )),
        ike_phase_1=dict(
            type="dict",
            options=dict(
                data_integrity=dict(
                    type="str",
                    choices=["aes-xcbc", "sha1", "sha256", "sha384", "sha512", "md5"],
                ),
                diffie_hellman_group=dict(
                    type="str",
                    choices=[
                        "group-1",
                        "group-2",
                        "group-5",
                        "group-14",
                        "group-19",
                        "group-20",
                    ],
                ),
                encryption_algorithm=dict(
                    type="str",
                    choices=["cast", "aes-256", "des", "aes-128", "3des"],
                ),
                ike_p1_rekey_time=dict(type='int', no_log=False),
                ike_p1_rekey_time_unit=dict(type='str', choices=['days', 'hours', 'minutes', 'seconds']),
            ),
        ),
        ike_phase_2=dict(
            type="dict",
            options=dict(
                data_integrity=dict(
                    type="str",
                    choices=["aes-xcbc", "sha1", "sha256", "sha384", "sha512", "md5"],
                ),
                encryption_algorithm=dict(
                    type="str",
                    choices=[
                        "cast",
                        "aes-gcm-256",
                        "cast-40",
                        "aes-256",
                        "des",
                        "aes-128",
                        "3des",
                        "des-40cp",
                        "aes-gcm-128",
                        "none",
                    ],
                ),
                ike_p2_use_pfs=dict(type='bool'),
                ike_p2_pfs_dh_grp=dict(type='str', choices=['group-1', 'group-2', 'group-5', 'group-14',
                                                            'group-15', 'group-16', 'group-17', 'group-18', 'group-19',
                                                            'group-20', 'group-24']),
                ike_p2_rekey_time=dict(type='int', no_log=False),
                ike_p2_rekey_time_unit=dict(type='str', choices=['days', 'hours', 'minutes', 'seconds']),
            ),
        ),
        mep=dict(
            type="dict",
            options=dict(
                enabled=dict(
                    type="bool"
                ),
                entry_point_selection_mechanism=dict(
                    type="str",
                    choices=[
                        "closest gateway to source",
                        "closest gateway to destination",
                        "random selection",
                        "manual"
                    ]
                ),
                entry_point_final_selection_mechanism=dict(
                    type="str",
                    choices=[
                        "random selection",
                        "first to respond"
                    ]
                ),
                tracking=dict(
                    type="str",
                    choices=[
                        "none",
                        "log",
                        "popup alert",
                        "mail alert",
                        "snmp trap alert",
                        "user defined alert no.1",
                        "user defined alert no.2",
                        "user defined alert no.3"
                    ]
                ),
                default_priority_rule=dict(
                    type="dict",
                    options=dict(
                        first_priority_center_gateways=dict(
                            type="list", elements="str"
                        ),
                        second_priority_center_gateways=dict(
                            type="list", elements="str"
                        ),
                        third_priority_center_gateways=dict(
                            type="list", elements="str"
                        )
                    )
                ),
                exception_priority_rules=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        satellite_gateways=dict(
                            type="list", elements="str"
                        ),
                        first_priority_center_gateways=dict(
                            type="list", elements="str"
                        ),
                        second_priority_center_gateways=dict(
                            type="list", elements="str"
                        ),
                        third_priority_center_gateways=dict(
                            type="list", elements="str"
                        )
                    )
                )
            )
        ),
        mesh_center_gateways=dict(type="bool"),
        permanent_tunnels=dict(
            type="dict",
            options=dict(
                set_permanent_tunnels=dict(
                    type="str",
                    choices=[
                        "on all tunnels in the community",
                        "on all tunnels of specific gateways",
                        "on specific tunnels in the community",
                        "off"
                    ]
                ),
                gateways=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        gateway=dict(
                            type="str"
                        ),
                        track_options=dict(
                            type="str",
                            choices=[
                                "according to community track options",
                                "override track options"
                            ]
                        ),
                        override_tunnel_down_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3"
                            ]
                        ),
                        override_tunnel_up_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3"
                            ]
                        )
                    )
                ),
                tunnels=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        first_tunnel_endpoint=dict(
                            type="str"
                        ),
                        second_tunnel_endpoint=dict(
                            type="str"
                        ),
                        track_options=dict(
                            type="str",
                            choices=[
                                "according to community track options",
                                "override track options"
                            ]
                        ),
                        override_tunnel_down_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3"
                            ]
                        ),
                        override_tunnel_up_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3"
                            ]
                        )
                    )
                ),
                rim=dict(
                    type="dict",
                    options=dict(
                        enabled=dict(
                            type="bool"
                        ),
                        enable_on_center_gateways=dict(
                            type="bool"
                        ),
                        enable_on_satellite_gateways=dict(
                            type="bool"
                        ),
                        route_injection_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3"
                            ]
                        )
                    )
                ),
                tunnel_down_track=dict(
                    type="str",
                    choices=[
                        "none",
                        "log",
                        "popup alert",
                        "mail alert",
                        "snmp trap alert",
                        "user defined alert no.1",
                        "user defined alert no.2",
                        "user defined alert no.3"
                    ]
                ),
                tunnel_up_track=dict(
                    type="str",
                    choices=[
                        "none",
                        "log",
                        "popup alert",
                        "mail alert",
                        "snmp trap alert",
                        "user defined alert no.1",
                        "user defined alert no.2",
                        "user defined alert no.3"
                    ]
                )
            )
        ),
        override_vpn_domains=dict(type='list', elements="dict", options=dict(
            gateway=dict(type='str'),
            vpn_domain=dict(type='str')
        )),
        satellite_gateways=dict(type="list", elements="str"),
        shared_secrets=dict(
            type="list",
            elements="dict",
            no_log=True,
            options=dict(
                external_gateway=dict(type="str"),
                shared_secret=dict(type="str", no_log=True),
            ),
        ),
        tags=dict(type="list", elements="str"),
        tunnel_granularity=dict(type='str', choices=['per_host', 'per_subnet', 'universal']),
        use_shared_secret=dict(type="bool"),
        vpn_routing=dict(
            type="str",
            choices=[
                "to center only",
                "to center and to other satellites",
                "to center other satellites and internet"
            ]
        ),

        wire_mode=dict(
            type="dict",
            options=dict(
                allow_uninspected_encrypted_traffic=dict(
                    type="bool"
                ),
                allow_uninspected_encrypted_routing=dict(
                    type="bool"
                )
            )
        ),

        routing_mode=dict(
            type="str",
            choices=[
                "domain_based",
                "route_based"
            ]
        ),

        advanced_properties=dict(
            type="dict",
            options=dict(
                support_ip_compression=dict(
                    type="bool"
                ),
                use_aggressive_mode=dict(
                    type="bool"
                )
            )
        ),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "vpn-community-star"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_registry
version_added: "3.19.0"
short_description: Create, Update or Delete an Azure Machine Learning Registry
description:
    - Create, Update or Delete an Azure Machine Learning Registry.
options:
    name:
        description:
            - Name of the Azure ML registry.
        type: str
        required: true
    display_name:
        description:
            - Display name for the registry.
        type: str
        required: false
    resource_definition:
        description:
            - Registry definition in YAML
        type: str
        required: false
    public_network_access:
        description:
            - Allow public endpoint connectivity when a registry
              is private link enabled.
        type: str
        choices:
            - Disabled
            - Enabled
        default: Disabled
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    state:
        description:
            - State of the Registry. Use C(present) to create
              or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Registry
  azure.azcollection.azure_rm_ml_registry:
    name: MyRegistry
    description: My Registry created by Ansible
    resource_group: myResourceGroup
    resource_definition: |
      name: MyRegistry
      tags:
        foo: bar
      location: eastus
      replication_locations:
        - location: eastus
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_registry:
    description:
        - Registry that was just created or updated.
    returned: always
    type: dict
    sample: {
        "container_registry": {
            "acr_account_sku": "premium",
            "arm_resource_id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/azureml-rg-registry-xxxxxxx-xxx_xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.ContainerRegistry/registries/xxxxxxxxxxx"
        },
        "discovery_url": "https://eastus.api.azureml.ms/registrymanagement/v1.0/registries/registry-xxxxxxx-xxx/discovery",
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-registry/providers/Microsoft.MachineLearningServices/registries/registry-xxxxxxx-xxx",
        "identity": {
            "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "type": "system_assigned"
        },
        "location": "eastus",
        "managed_resource_group": "{'additional_properties': {}, 'resource_id': '/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/azureml-rg-registry-xxxxxxx-xxx_xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx'}",
        "mlflow_registry_uri": "azureml://eastus.api.azureml.ms/mlflow/v1.0/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-registry/providers/Microsoft.MachineLearningServices/registries/registry-xxxxxxx-xxx",
        "name": "registry-xxxxxxx-xxx",
        "public_network_access": "Disabled",
        "replication_locations": [
            {
                "location": "eastus",
                "storage_config": {
                    "arm_resource_id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/azureml-rg-registry-xxxxxxx-xxx_xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.Storage/storageAccounts/xxxxxxxxxxx",
                    "replication_count": 1,
                    "storage_account_hns": false,
                    "storage_account_type": "standard_lrs"
                }
            }
        ],
        "tags": {
            "foo": "bar"
        }
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_registry
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLRegistry(MLClientCommon):

    PARAM_OVERRIDES = ['name',
                       'display_name',
                       'tags',
                       'public_network_access',
                       ]

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True
            ),
            display_name=dict(
                type='str',
                required=False,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            public_network_access=dict(
                type='str',
                choices=['Disabled', 'Enabled'],
                default='Disabled',
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
        )

        self._client = None
        self.ml_registry = None
        self.ml_workspace = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_registry={}
        )

        super(AzureRMMLRegistry, self).__init__(self.module_arg_spec,
                                                supports_check_mode=True,
                                                )

    def exec_module(self, **kwargs):

        params_override = []
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])
            if kwargs[key] is not None and key in self.PARAM_OVERRIDES:
                params_override.append({key: kwargs[key]})

        if self.resource_definition:
            resource_definition = io.StringIO(self.resource_definition)
        else:
            resource_definition = None

        changed = False

        try:
            # Attempt to retrieve ml_registry based on name
            ml_registry_info = self.client.registries.get(self.name)
        except ResourceNotFoundError:
            ml_registry_info = None

        if self.state == 'present':
            # Generate ml_registry based on ansible args passed in.
            ml_registry = load_registry(resource_definition,
                                        params_override=params_override)
            if ml_registry_info:
                # Update
                ml_registry_info = self.entity_to_dict(ml_registry_info)
                changed = not self.default_compare({},
                                                   self.entity_to_dict(ml_registry),
                                                   ml_registry_info,
                                                   '',
                                                   self.results)
                if changed and not self.check_mode:
                    response = self.client.registries.begin_create(
                        registry=ml_registry
                    )
                    ml_registry_object = self.get_poller_result(response)
                    ml_registry_info = self.entity_to_dict(ml_registry_object)

            else:
                # Create
                changed = True
                if not self.check_mode:
                    response = self.client.registries.begin_create(
                        registry=ml_registry
                    )
                    ml_registry_object = self.get_poller_result(response)
                    ml_registry_info = self.entity_to_dict(ml_registry_object)
        else:
            # Delete
            if ml_registry_info:
                changed = True
                if not self.check_mode:
                    response = self.client.registries.begin_delete(
                        name=self.name
                    )
                    ml_registry_object = self.get_poller_result(response)
                    ml_registry_info = self.entity_to_dict(ml_registry_object)

        self.results['ml_registry'] = ml_registry_info
        self.results['changed'] = changed
        return self.results


def main():
    AzureRMMLRegistry()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2021, Werner Dijkerman (ikben@werner-dijkerman.nl)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: gitlab_protected_branch
short_description: Manage protection of existing branches
version_added: 3.4.0
description:
  - (un)Marking existing branches for protection.
author:
  - "Werner Dijkerman (@dj-wasabi)"
requirements:
  - python-gitlab >= 2.3.0
extends_documentation_fragment:
  - community.general._auth_basic
  - community.general._gitlab
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  state:
    description:
      - Create or delete protected branch.
    default: present
    type: str
    choices: ["present", "absent"]
  project:
    description:
      - The path and name of the project.
    required: true
    type: str
  name:
    description:
      - The name of the branch that needs to be protected.
      - Can make use a wildcard character for like V(production/*) or just have V(main) or V(develop) as value.
    required: true
    type: str
  merge_access_levels:
    description:
      - Access levels allowed to merge.
    default: maintainer
    type: str
    choices: ["maintainer", "developer", "nobody"]
  push_access_level:
    description:
      - Access levels allowed to push.
    default: maintainer
    type: str
    choices: ["maintainer", "developer", "nobody"]
  allow_force_push:
    description:
      - Whether or not to allow force pushes to the protected branch.
    type: bool
    version_added: '11.3.0'
  code_owner_approval_required:
    description:
      - Whether or not to require code owner approval to push.
    type: bool
    version_added: '11.3.0'
"""


EXAMPLES = r"""
- name: Create protected branch on main
  community.general.gitlab_protected_branch:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "dj-wasabi/collection.general"
    name: main
    merge_access_levels: maintainer
    push_access_level: nobody
"""

RETURN = r"""
"""

from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._gitlab import (
    auth_argument_spec,
    gitlab,
    gitlab_authentication,
)
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


class GitlabProtectedBranch:
    def __init__(self, module, project, gitlab_instance):
        self.repo = gitlab_instance
        self._module = module
        self.project = self.get_project(project)
        self.ACCESS_LEVEL = {
            "nobody": gitlab.const.NO_ACCESS,
            "developer": gitlab.const.DEVELOPER_ACCESS,
            "maintainer": gitlab.const.MAINTAINER_ACCESS,
        }

    def get_project(self, project_name):
        return self.repo.projects.get(project_name)

    def protected_branch_exist(self, name):
        try:
            return self.project.protectedbranches.get(name)
        except Exception:
            return False

    def create_or_update_protected_branch(self, name, options):
        protected_branch_options = {
            "name": name,
            "allow_force_push": options["allow_force_push"],
            "code_owner_approval_required": options["code_owner_approval_required"],
        }
        protected_branch = self.protected_branch_exist(name=name)
        changed = False
        if protected_branch and self.can_update(protected_branch, options):
            for arg_key, arg_value in protected_branch_options.items():
                if arg_value is not None:
                    if getattr(protected_branch, arg_key) != arg_value:
                        setattr(protected_branch, arg_key, arg_value)
                        changed = True
            if changed and not self._module.check_mode:
                protected_branch.save()
        else:
            # Set immutable options only on (re)creation
            protected_branch_options["merge_access_level"] = options["merge_access_levels"]
            protected_branch_options["push_access_level"] = options["push_access_level"]
            if protected_branch:
                # Exists, but couldn't update. So, delete first
                self.delete_protected_branch(name)
            if not self._module.check_mode:
                self.project.protectedbranches.create(protected_branch_options)
            changed = True

        return changed

    def can_update(self, protected_branch, options):
        # these keys are not set on update the same way they are on creation
        configured_merge = options["merge_access_levels"]
        configured_push = options["push_access_level"]
        current_merge = protected_branch.merge_access_levels[0]["access_level"]
        current_push = protected_branch.push_access_levels[0]["access_level"]
        return (configured_merge is None or current_merge == configured_merge) and (
            configured_push is None or current_push == configured_push
        )

    def delete_protected_branch(self, name):
        if self._module.check_mode:
            return True
        return self.project.protectedbranches.delete(name)


def main():
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(auth_argument_spec())
    argument_spec.update(
        project=dict(type="str", required=True),
        name=dict(type="str", required=True),
        merge_access_levels=dict(type="str", default="maintainer", choices=["maintainer", "developer", "nobody"]),
        push_access_level=dict(type="str", default="maintainer", choices=["maintainer", "developer", "nobody"]),
        allow_force_push=dict(type="bool"),
        code_owner_approval_required=dict(type="bool"),
        state=dict(type="str", default="present", choices=["absent", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["api_username", "api_token"],
            ["api_username", "api_oauth_token"],
            ["api_username", "api_job_token"],
            ["api_token", "api_oauth_token"],
            ["api_token", "api_job_token"],
        ],
        required_together=[
            ["api_username", "api_password"],
        ],
        required_one_of=[["api_username", "api_token", "api_oauth_token", "api_job_token"]],
        supports_check_mode=True,
    )

    # check prerequisites and connect to gitlab server
    gitlab_instance = gitlab_authentication(module)

    project = module.params["project"]
    name = module.params["name"]
    merge_access_levels = module.params["merge_access_levels"]
    push_access_level = module.params["push_access_level"]
    state = module.params["state"]

    gitlab_version = gitlab.__version__
    if LooseVersion(gitlab_version) < LooseVersion("2.3.0"):
        module.fail_json(
            msg=f"community.general.gitlab_protected_branch requires python-gitlab Python module >= 2.3.0 (installed version: [{gitlab_version}])."
            " Please upgrade python-gitlab to version 2.3.0 or above."
        )

    this_gitlab = GitlabProtectedBranch(module=module, project=project, gitlab_instance=gitlab_instance)

    p_branch = this_gitlab.protected_branch_exist(name=name)
    options = {
        "merge_access_levels": this_gitlab.ACCESS_LEVEL[merge_access_levels],
        "push_access_level": this_gitlab.ACCESS_LEVEL[push_access_level],
        "allow_force_push": module.params["allow_force_push"],
        "code_owner_approval_required": module.params["code_owner_approval_required"],
    }
    if state == "present":
        changed = this_gitlab.create_or_update_protected_branch(name, options)
        module.exit_json(changed=changed, msg="Created or updated the protected branch.")
    elif p_branch and state == "absent":
        this_gitlab.delete_protected_branch(name=name)
        module.exit_json(changed=True, msg="Deleted the protected branch.")
    module.exit_json(changed=False, msg="No changes are needed.")


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2025 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_keyvaultcertificate
version_added: "3.4.0"
short_description: Managed keyvault certificate
description:
    - Managed keyvault certificate.

options:
    vault_uri:
        description:
            - Vault uri where the certificate stored in.
        required: True
        type: str
    name:
        description:
            - Certificate name.
        type: str
        required: True
    policy:
        description:
            - The management policy for the certificate.
            - When generating a new certificate, if no policy is set, the default policy will be used.
        type: dict
        suboptions:
            subject:
                description:
                    - The subject name of the certificate. Should be a valid X509 distinguished name.
                    - Either subject or one of the subject alternative name parameters are required for creating a certificate.
                    - This will be ignored when importing a certificate; the subject will be parsed from the imported certificate.
                type: str
            issuer_name:
                description:
                    - Name of the referenced issuer object or reserved names. For example C(self) and C(unknown).
                type: str
                choices:
                    - self
                    - unknown
            exportable:
                description:
                    - Indicates if the private key can be exported. For valid values, see KeyType.
                type: bool
            key_type:
                description:
                    - The type of key pair to be used for the certificate.
                type: str
                choices:
                    - EC
                    - EC-HSM
                    - RSA
                    - RSA-HSM
                    -  oct
                    - oct-HSM
            key_size:
                description:
                    - The key size in bits. For example C(2048), C(3072), or C(4096) for RSA.
                type: int
            reuse_key:
                description:
                    - Indicates if the same key pair will be used on certificate renewal.
                type: bool
            key_curve_name:
                description:
                    - Elliptic curve name. For valid values, see KeyCurveName.
                type: str
                choices:
                    - P-256
                    - P-384
                    - P-521
                    - P-256K
            enhanced_key_usage:
                description:
                    - The extended ways the key of the certificate can be used.
                type: list
                elements: str
            content_type:
                description:
                    - he media type (MIME type) of the secret backing the certificate.
                type: str
                default: application/x-pkcs12
                choices:
                    - application/x-pkcs12
                    - application/x-pem-file
            key_usage:
                description:
                    - The extended ways the key of the certificate can be used.
                type: list
                elements: str
                choices:
                    - digitalSignature
                    - nonRepudiation
                    - keyEncipherment
                    - dataEncipherment
                    - keyAgreement
                    - keyCertSign
                    - cRLSign
                    - encipherOnly
                    - decipherOnly
            validity_in_months:
                description:
                    - The duration that the certificate is valid in months.
                type: int
            lifetime_actions:
                description:
                    - Actions that will be performed by Key Vault over the lifetime of a certificate.
                type: list
                elements: dict
                suboptions:
                    action:
                        description:
                            - The type of the action.
                        type: str
                        choices:
                            - EmailContacts
                            - AutoRenew
                    lifetime_percentage:
                        description:
                            - Percentage of lifetime at which to trigger. Value should be between 1 and 99.
                        type: int
                    days_before_expiry:
                        description:
                            - Days before expiry to attempt renewal.
                            - Value should be between 1 and `validity_in_months` multiplied by 27.
                            - If validity_in_months is 36, then value should be between 1 and 972 (36 * 27).
                        type: int
            certificate_type:
                description:
                    - Type of certificate to be requested from the issuer provider.
                type: str
            san_emails:
                description:
                    - Subject alternative emails of the X509 object.
                    - Either subject or one of the subject alternative name parameters are required for creating a certificate.
                type: list
                elements: str
            certificate_transparency:
                description:
                    - Indicates if the certificates generated under this policy should be published to certificate transparency logs.
                type: bool
            san_dns_names:
                description:
                    - Subject alternative DNS names of the X509 object.
                    - Either subject or one of the subject alternative name parameters are required for creating a certificate.
                type: list
                elements: str
            san_user_principal_names:
                description:
                    - Subject alternative user principal names of the X509 object.
                    - Either subject or one of the subject alternative name parameters are required for creating a certificate.
                type: list
                elements: str
    enabled:
        description:
            - Whether the certificate is enabled for use.
        type: bool
    password:
        description:
            - If the private key in the passed in certificate is encrypted, it is the password used for encryption.
        type: str
    cert_data:
        description:
            - Aan existing valid certificate, containing a private key, into Azure Key Vault.
        type: str
    state:
        description:
            - State of the keyvault certificate.
        type: str
        required: True
        choices:
            - generate
            - import
            - delete
            - purge
            - recover
            - merge
            - update

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Import a keyvault certificate
  azure_rm_keyvaultcertificate:
    vault_uri: https://vault{{ rpfx }}.vault.azure.net
    name: fredcerticate
    enabled: true
    password: Password@****
    cert_data: "{{ lookup('file', 'cert.pem') }}"
    state: import
    tags:
      key1: value1

- name: Generate a keyvault certificate
  azure_rm_keyvaultcertificate:
    vault_uri: https://vault{{ rpfx }}.vault.azure.net
    name: fredcerticate
    policy:
      subject: 'CN=Anhui02'
      issuer_name: self
      exportable: true
      key_type: RSA
      key_size: 2048
      san_emails:
        - 7170222076@qq.com
      content_type: 'application/x-pkcs12'
      validity_in_months: 36
      lifetime_actions:
        - action: EmailContacts
          days_before_expiry: 10
    enabled: true
    state: generate

- name: Rotate Key Vault Certificate with a renewed PEM bundle
  azure_rm_keyvaultcertificate:
    vault_uri: https://vault{{ rpfx }}.vault.azure.net
    name: fredcerticate
    cert_data: "{{ lookup('file', 'new_cert.pem') }}" # PEM must include private key + cert
    policy:
      content_type: "application/x-pem-file"
    state: import

- name: Rotate Key Vault Certificate with a renewed PFX bundle
  azure_rm_keyvaultcertificate:
    vault_uri: https://vault{{ rpfx }}.vault.azure.net
    name: fredcerticate
    cert_data: "{{ lookup('file', 'new_cert.pfx') | b64encode }}" # PFX must include private key + cert
    password: Password@****
    policy:
      content_type: "application/x-pkcs12"
    state: import

- name: Update the keyvault certificate
  azure_rm_keyvaultcertificate:
    vault_uri: https://vault{{ rpfx }}.vault.azure.net
    name: fredcerticate
    policy:
      subject: 'CN=Anhui'
      issuer_name: self
      exportable: true
      key_type: RSA
      key_size: 2048
      san_emails:
        - 7170222076@qq.com
      content_type: 'application/x-pkcs12'
      validity_in_months: 36
      lifetime_actions:
        - action: EmailContacts
          days_before_expiry: 10
    enabled: true
    state: update

- name: Purge the keyvault certificate
  azure_rm_keyvaultcertificate:
    vault_uri: https://vault{{ rpfx }}.vault.azure.net
    name: fredcerticate
    state: purge

- name: Recover the keyvault certificate
  azure_rm_keyvaultcertificate:
    vault_uri: https://vault{{ rpfx }}.vault.azure.net
    name: fredcerticate
    state: recover

- name: Delete the keyvault certificate
  azure_rm_keyvaultcertificate:
    vault_uri: https://vault{{ rpfx }}.vault.azure.net
    name: fredcerticate
    state: absent
'''

RETURN = '''
certificates:
    description:
        - The facts of certificates in Azure Key Vault.
    returned: always
    type: complex
    contains:
        cert_data:
            description:
                - CER contents of the X509 certificate.
            type: str
            returned: always
            sample: "MIID*****************0pRjXE"
        name:
            description:
                - The name of the certificate.
            type: str
            returned: always
            sample: testcert
        deleted_on:
            description:
                - The time when the certificate was deleted, in UTC.
            returned: always
            type: str
            sample: 2025-01-14T09"
        recovery_id:
            description:
                - The url of the recovery object, used to identify and recover the deleted certificate.
            type: str
            returned: always
            sample: "https://vaultrfred01.vault.azure.net/deletedcertificates/cert02"
        scheduled_purge_date:
            description:
                - The time when the certificate is scheduled to be purged, in UTC.
            returned: always
            type: dict
            sample: 2025-02-14T09"
        policy:
            description:
                - The management policy of the deleted certificate.
            returned: always
            type: complex
            contains:
                attributes:
                    description:
                        - Certificate attributes.
                    type: complex
                    returned: always
                    contains:
                        created:
                            description:
                                - Creation datetime.
                            returned: always
                            type: str
                            sample: "2025-01-14T09:41:20+00:00"
                        not_before:
                            description:
                                - Not before datetime.
                            type: str
                            sample: None
                        expires:
                            description:
                                - Expiration datetime.
                            type: str
                            sample: None
                        updated:
                            description:
                                - Update datetime.
                            returned: always
                            type: str
                            sample: "2025-01-15T09:41:20+00:00"
                        enabled:
                            description:
                                - Indicate whether the certificate is enabled.
                            returned: always
                            type: str
                            sample: true
                        recovery_level:
                            description:
                                - Reflects the deletion recovery level currently in effect for certificates in the current vault.
                                - If it contains 'Purgeable' the certificate can be permanently deleted by a privileged user,
                                - Otherwise, only the system can purge the certificate, at the end of the retention interval.
                            returned: always
                            type: str
                            sample: None
                        recoverable_days:
                            description:
                                - Reflects the deletion recovery days.
                            type: int
                            returned: always
                            sample: None
                issuer_name:
                    description:
                        - Name of the referenced issuer object or reserved names.
                    type: str
                    returned: always
                    sample: Self
                subject:
                    description:
                        - The subject name of the certificate.
                        - Should be a valid X509 distinguished name.
                        - Either subject or one of the subject alternative name parameters are required for creating a certificate.
                        - This will be ignored when importing a certificate; the subject will be parsed from the imported certificate.
                    type: str
                    returned: always
                    sample: CN=anhui.com
                san_emails:
                    description:
                        - Subject alternative emails of the X509 object.
                        - Either subject or one of the subject alternative name parameters are required for creating a certificate.
                    type: str
                    returned: always
                    sample: None
                san_dns_names:
                    description:
                        - Subject alternative DNS names of the X509 object.
                        - Either subject or one of the subject alternative name parameters are required for creating a certificate.
                    type: str
                    returned: always
                    sample: None
                san_user_principal_names:
                    description:
                        - Subject alternative user principal names of the X509 object.
                        - Either subject or one of the subject alternative name parameters are required for creating a certificate.
                    type: str
                    returned: always
                    sample: None
                exportable:
                    description:
                        - Indicates if the private key can be exported.
                    type: bool
                    returned: always
                    sample: true
                key_type:
                    description:
                        - The type of key pair to be used for the certificate.
                    type: str
                    returned: always
                    sample: RSA
                key_size:
                    description:
                        - The key size in bits.
                    type: int
                    returned: always
                    sample: 2048
                reuse_key:
                    description:
                        - Indicates if the same key pair will be used on certificate renewal.
                    type: bool
                    returned: always
                    sample: false
                key_curve_name:
                    description:
                        - Elliptic curve name. For valid values, see KeyCurveName.
                    type: str
                    returned: always
                    sample: None
                enhanced_key_usage:
                    description:
                        - The extended ways the key of the certificate can be used.
                    type: list
                    returned: always
                    sample: ['1.3.6.1.5.5.7.3.1', '1.3.6.1.5.5.7.3.2']
                key_usage:
                    description:
                        - List of key usages.
                    type: list
                    returned: always
                    sample: ["digitalSignature", "keyEncipherment"]
                content_type:
                    description:
                        - If not specified, the media type (MIME type) of the secret backing the certificate.
                    type: str
                    returned: always
                    sample: application/x-pkcs12
                validity_in_months:
                    description:
                        - The duration that the certificate is valid in months.
                    type: int
                    returned: always
                    sample: 12
                lifetime_actions:
                    description:
                        - Actions that will be performed by Key Vault over the lifetime of a certificate.
                    type: list
                    returned: always
                    sample: [{'action': 'AutoRenew', 'days_before_expiry': None, 'lifetime_percentage': 80}]
                certificate_type:
                    description:
                        - Type of certificate to be requested from the issuer provider.
                    type: str
                    returned: str
                    sample: None
                certificate_transparency:
                    description:
                        - Indicates if the certificates generated under this policy should be published to certificate transparency logs.
                    type: bool
                    returned: always
                    sample: None
        properties:
            description:
                - The certificate's properties.
            type: complex
            returned: always
            contains:
                id:
                    description:
                        - Id of the certificate. If specified all other 'Id' arguments should be omitted.
                    type: str
                    returned: always
                    sample: "https://vaultrfred01.vault.azure.net/certificates/cert02/62409e6304c642f193209729b8360d2c"
                vault_id:
                    description:
                        - ID of the Key Vault.
                    type: str
                    returned: always
                    sample: "https://vaultrfred01.vault.azure.net"
                x509_thumbprint:
                    description:
                        - The X509 Thumbprint of the Key Vault Certificate represented as a hexadecimal string.
                    type: str
                    returned: always
                    sample: 1blAnHN9ddng0qh1pYoUDY2lp1E=
                tags:
                    description:
                        - List of the certificate tags.
                    type: dict
                    returned: always
                    sample: {'key': 'value'}
                attributes:
                    description:
                        - Certificate attributes.
                    type: complex
                    returned: always
                    contains:
                        created:
                            description:
                                - Creation datetime.
                            returned: always
                            type: str
                            sample: "2025-01-14T09"
                        not_before:
                            description:
                                - Not before datetime.
                            type: str
                            sample: "2025-02-14T09"
                        expires:
                            description:
                                - Expiration datetime.
                            type: str
                            sample: "2025-03-14T09"
                        updated:
                            description:
                                - Update datetime.
                            returned: always
                            type: str
                            sample: "2025-01-15T09"
                        enabled:
                            description:
                                - Indicate whether the certificate is enabled.
                            returned: always
                            type: str
                            sample: true
                        recovery_level:
                            description:
                                - Reflects the deletion recovery level currently in effect for certificates in the current vault.
                                - If it contains 'Purgeable' the certificate can be permanently deleted by a privileged user,
                                - Otherwise, only the system can purge the certificate, at the end of the retention interval.
                            returned: always
                            type: str
                            sample: Recoverable+Purgeable
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.keyvault.certificates import CertificateClient, CertificatePolicy, LifetimeAction
    import base64
    import hashlib
    try:
        from cryptography.hazmat.primitives import hashes
        from cryptography.hazmat.primitives.serialization import pkcs12
    except Exception:
        hashes = None
        pkcs12 = None
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


def certificatebundle_to_dict(certificate):
    response = dict(policy=dict(), properties=dict(), cert_data=None)
    if certificate.cer is not None:
        response['cert_data'] = base64.b64encode(certificate.cer).decode('utf-8')
    response['name'] = certificate.name
    if certificate.policy is not None:
        response['policy']['issuer_name'] = certificate.policy._issuer_name
        response['policy']['subject'] = certificate.policy._subject
        response['policy']['exportable'] = certificate.policy._exportable
        response['policy']['key_type'] = certificate.policy._key_type
        response['policy']['key_size'] = certificate.policy._key_size
        response['policy']['reuse_key'] = certificate.policy._reuse_key
        response['policy']['key_curve_name'] = certificate.policy._key_curve_name
        response['policy']['enhanced_key_usage'] = certificate.policy._enhanced_key_usage
        response['policy']['key_usage'] = certificate.policy._key_usage
        response['policy']['content_type'] = certificate.policy._content_type
        response['policy']['validity_in_months'] = certificate.policy._validity_in_months
        response['policy']['certificate_type'] = certificate.policy._certificate_type
        response['policy']['certificate_transparency'] = certificate.policy._certificate_transparency
        response['policy']['san_emails'] = certificate.policy._san_emails
        response['policy']['san_dns_names'] = certificate.policy._san_dns_names
        response['policy']['san_user_principal_names'] = certificate.policy._san_user_principal_names
        response['policy']['attributes'] = dict()
        if certificate.policy._attributes is not None:
            response['policy']['attributes']['enabled'] = certificate.policy._attributes.enabled
            response['policy']['attributes']['not_before'] = certificate.policy._attributes.not_before
            response['policy']['attributes']['expires'] = certificate.policy._attributes.expires
            response['policy']['attributes']['created'] = certificate.policy._attributes.created
            response['policy']['attributes']['updated'] = certificate.policy._attributes.updated
            response['policy']['attributes']['recoverable_days'] = certificate.policy._attributes.recoverable_days
            response['policy']['attributes']['recovery_level'] = certificate.policy._attributes.recovery_level
        else:
            response['policy']['attributes'] = None
        if certificate.policy._lifetime_actions is not None:
            response['policy']['lifetime_actions'] = []
            for item in certificate.policy._lifetime_actions:
                response['policy']['lifetime_actions'].append(dict(action=item.action,
                                                                   lifetime_percentage=item.lifetime_percentage,
                                                                   days_before_expiry=item.days_before_expiry))
        else:
            response['policy']['lifetime_actions'] = None
    else:
        response['policy'] = None

    if certificate.properties is not None:
        response['properties']['attributes'] = dict(enabled=certificate.properties._attributes.enabled,
                                                    not_before=certificate.properties._attributes.not_before,
                                                    expires=certificate.properties._attributes.expires,
                                                    created=certificate.properties._attributes.created,
                                                    updated=certificate.properties._attributes.updated,
                                                    recovery_level=certificate.properties._attributes.recovery_level)
        response['properties']['id'] = certificate.properties._id
        response['properties']['vault_id'] = certificate.properties._vault_id.vault_url if certificate.properties._vault_id is not None else None
        response['properties']['x509_thumbprint'] = base64.b64encode(certificate.properties._x509_thumbprint).decode('utf-8')
        response['properties']['tags'] = certificate.properties._tags
    else:
        response['properties'] = None

    return response


def policy_bundle_to_dict(policy):
    result = dict()
    if policy is not None:
        result['issuer_name'] = policy._issuer_name
        result['subject'] = policy._subject
        result['exportable'] = policy._exportable
        result['key_type'] = policy._key_type
        result['key_size'] = policy._key_size
        result['reuse_key'] = policy._reuse_key
        result['key_curve_name'] = policy._key_curve_name
        result['enhanced_key_usage'] = policy._enhanced_key_usage
        result['key_usage'] = policy._key_usage
        result['content_type'] = policy._content_type
        result['validity_in_months'] = policy._validity_in_months
        result['certificate_type'] = policy._certificate_type
        result['certificate_transparency'] = policy._certificate_transparency
        result['san_emails'] = policy._san_emails
        result['san_dns_names'] = policy._san_dns_names
        result['san_user_principal_names'] = policy._san_user_principal_names
        result['attributes'] = dict()
        if policy._attributes is not None:
            result['attributes']['enabled'] = policy._attributes.enabled
            result['attributes']['not_before'] = policy._attributes.not_before
            result['attributes']['expires'] = policy._attributes.expires
            result['attributes']['created'] = policy._attributes.created
            result['attributes']['updated'] = policy._attributes.updated
            result['attributes']['recoverable_days'] = policy._attributes.recoverable_days
            result['attributes']['recovery_level'] = policy._attributes.recovery_level
        else:
            result['attributes'] = None
        if policy._lifetime_actions is not None:
            result['lifetime_actions'] = []
            for item in policy._lifetime_actions:
                result['lifetime_actions'].append(dict(action=item.action,
                                                       lifetime_percentage=item.lifetime_percentage,
                                                       days_before_expiry=item.days_before_expiry))
        else:
            result['lifetime_actions'] = None
    else:
        result = None

    return result


def deleted_certificatebundle_to_dict(certificate):
    response = dict(policy=dict(), properties=dict(), cert_data=None)
    if certificate.cer is not None:
        response['cert_data'] = str(certificate.cer)
    response['name'] = certificate.name
    response['recovery_id'] = certificate._recovery_id
    response['scheduled_purge_date'] = certificate._scheduled_purge_date
    response['deleted_on'] = certificate._deleted_on
    if certificate.cer is not None:
        response['cert_data'] = str(certificate.cer)
    if certificate.policy is not None:
        response['policy']['issuer_name'] = certificate.policy._issuer_name
        response['policy']['subject'] = certificate.policy._subject
        response['policy']['exportable'] = certificate.policy._exportable
        response['policy']['key_type'] = certificate.policy._key_type
        response['policy']['key_size'] = certificate.policy._key_size
        response['policy']['reuse_key'] = certificate.policy._reuse_key
        response['policy']['key_curve_name'] = certificate.policy._key_curve_name
        response['policy']['enhanced_key_usage'] = certificate.policy._enhanced_key_usage
        response['policy']['key_usage'] = certificate.policy._key_usage
        response['policy']['content_type'] = certificate.policy._content_type
        response['policy']['validity_in_months'] = certificate.policy._validity_in_months
        response['policy']['certificate_type'] = certificate.policy._certificate_type
        response['policy']['certificate_transparency'] = certificate.policy._certificate_transparency
        response['policy']['san_emails'] = certificate.policy._san_emails
        response['policy']['san_dns_names'] = certificate.policy._san_dns_names
        response['policy']['san_user_principal_names'] = certificate.policy._san_user_principal_names
        response['policy']['attributes'] = dict()
        if certificate.policy._attributes is not None:
            response['policy']['attributes']['enabled'] = certificate.policy._attributes.enabled
            response['policy']['attributes']['not_before'] = certificate.policy._attributes.not_before
            response['policy']['attributes']['expires'] = certificate.policy._attributes.expires
            response['policy']['attributes']['created'] = certificate.policy._attributes.created
            response['policy']['attributes']['updated'] = certificate.policy._attributes.updated
            response['policy']['attributes']['recoverable_days'] = certificate.policy._attributes.recoverable_days
            response['policy']['attributes']['recovery_level'] = certificate.policy._attributes.recovery_level
        else:
            response['policy']['attributes'] = None
        if certificate.policy._lifetime_actions is not None:
            response['policy']['lifetime_actions'] = []
            for item in certificate.policy._lifetime_actions:
                response['policy']['lifetime_actions'].append(dict(action=item.action,
                                                                   lifetime_percentage=item.lifetime_percentage,
                                                                   days_before_expiry=item.days_before_expiry))
        else:
            response['policy']['lifetime_actions'] = None
    else:
        response['policy'] = None

    if certificate.properties is not None:
        response['properties']['attributes'] = dict(enabled=certificate.properties._attributes.enabled,
                                                    not_before=certificate.properties._attributes.not_before,
                                                    expires=certificate.properties._attributes.expires,
                                                    created=certificate.properties._attributes.created,
                                                    updated=certificate.properties._attributes.updated,
                                                    recovery_level=certificate.properties._attributes.recovery_level)
        response['properties']['id'] = certificate.properties._id
        response['properties']['vault_id'] = certificate.properties._vault_id.vault_url if certificate.properties._vault_id is not None else None
        response['properties']['x509_thumbprint'] = base64.b64encode(certificate.properties._x509_thumbprint).decode('utf-8')
        response['properties']['tags'] = certificate.properties._tags
    else:
        response['properties'] = None
    return response


def _cert_data_thumbprint_sha1(cert_data_str, content_type=None, password=None):
    """
    Returns SHA-1 thumbprint (bytes) of the leaf certificate contained in cert_data_str.
    Supports PEM ('-----BEGIN CERTIFICATE-----') and PKCS12 (application/x-pkcs12) if cryptography is available.
    Returns None when parsing fails (caller should decide fallback behavior).
    """
    try:
        # PEM: extract first CERT block and hash its DER
        if "-----BEGIN CERTIFICATE-----" in cert_data_str:
            lines = cert_data_str.strip().splitlines()
            collecting = False
            b64 = []
            for ln in lines:
                if ln.startswith("-----BEGIN CERTIFICATE-----"):
                    collecting = True
                    b64 = []
                    continue
                if ln.startswith("-----END CERTIFICATE-----"):
                    collecting = False
                    der = base64.b64decode("".join(b64))
                    return hashlib.sha1(der).digest()
                if collecting:
                    b64.append(ln.strip())

        # PKCS#12 (PFX): parse via cryptography if available
        if content_type and content_type.lower() in ("application/x-pkcs12", "application/x-pfx"):
            if pkcs12 and hashes:
                try:
                    raw = base64.b64decode(cert_data_str)
                except Exception:
                    raw = cert_data_str.encode("utf-8", "ignore")
                pk = pkcs12.load_key_and_certificates(raw, None if password in (None, "") else password.encode("utf-8"))
                if pk and pk[1]:
                    return pk[1].fingerprint(hashes.SHA1())
            return None

        # Fallback: attempt base64->DER->SHA1
        try:
            der = base64.b64decode(cert_data_str)
            return hashlib.sha1(der).digest()
        except Exception:
            return None
    except Exception:
        return None


policy_spec = dict(
    issuer_name=dict(type='str', choices=['self', 'unknown']),
    subject=dict(type='str'),
    exportable=dict(type='bool',),
    key_type=dict(type='str', choices=['EC', 'EC-HSM', 'RSA', 'RSA-HSM', 'oct', 'oct-HSM']),
    key_size=dict(type='int'),
    reuse_key=dict(type='bool',),
    key_curve_name=dict(type='str', choices=['P-256', 'P-384', 'P-521', 'P-256K']),
    enhanced_key_usage=dict(type='list', elements='str'),
    key_usage=dict(
        type='list',
        elements='str',
        choices=['digitalSignature', 'nonRepudiation', 'keyEncipherment', 'dataEncipherment',
                 'keyAgreement', 'keyCertSign', 'cRLSign', 'encipherOnly', 'decipherOnly']
    ),
    content_type=dict(type='str', default='application/x-pkcs12', choices=['application/x-pkcs12', 'application/x-pem-file']),
    validity_in_months=dict(type='int',),
    certificate_type=dict(type='str'),
    certificate_transparency=dict(type='bool'),
    san_emails=dict(type='list', elements='str'),
    san_dns_names=dict(type='list', elements='str'),
    san_user_principal_names=dict(type='list', elements='str'),
    lifetime_actions=dict(
        type='list',
        elements='dict',
        options=dict(
            action=dict(type='str', choices=['EmailContacts', 'AutoRenew']),
            lifetime_percentage=dict(type='int'),
            days_before_expiry=dict(type='int')
        )
    ),
)


class AzureRMKeyVaultCertificate(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(name=dict(type='str', required=True),
                                    vault_uri=dict(type='str', required=True),
                                    policy=dict(type='dict', options=policy_spec),
                                    enabled=dict(type='bool'),
                                    password=dict(type='str', no_log=True),
                                    cert_data=dict(type='str', no_log=True),
                                    state=dict(
                                        type='str',
                                        required=True,
                                        choices=['generate', 'import', 'delete', 'purge', 'update', 'recover', 'merge']))
        self.vault_uri = None
        self.name = None
        self.policy = None
        self.enabled = None
        self.cert_data = None
        self.password = None
        self.state = None
        self.tags = None

        self.results = dict(changed=False)
        self._client = None
        required_if = [('state', 'import', ['cert_data'])]

        super(AzureRMKeyVaultCertificate,
              self).__init__(derived_arg_spec=self.module_arg_spec,
                             supports_check_mode=True,
                             supports_tags=True,
                             required_if=required_if,
                             facts_module=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            else:
                setattr(self, key, None)

        self._client = self.get_keyvault_client()
        changed = False
        response = None

        del_response = self.get_deleted_certificate()
        response = self.get_certificate()

        if self.state == 'import' and del_response is not None and response is None:
            self.fail("The certificate {0} is soft-deleted and recoverable; recover or purge before importing".format(self.name))

        if self.state == 'delete':
            if response is not None:
                changed = True
                if not self.check_mode:
                    response = self.delete_certificate()
        elif self.state == 'purge':
            if del_response is not None:
                changed = True
                if not self.check_mode:
                    response = self.purge_certificate()
        elif self.state == 'merge':
            if response is not None:
                changed = True
                if not self.check_mode:
                    response = self.merge_certificate()
            else:
                self.fail("The certificate not exist {0}".format(self.name))
        elif self.state == 'recover':
            if del_response is not None:
                changed = True
                if not self.check_mode:
                    response = self.recover_certificate()
            else:
                self.log("The certificate {0} exist or purged".format(self.name))
        else:
            if response is not None:
                a = {}
                b = dict(compare=[])

                # if not self.default_compare({}, self.policy, response['policy'], '', dict(compare=[])):
                if not self.default_compare(a, self.policy, response['policy'], '', b):
                    changed = True
                    if not self.check_mode:
                        response['policy'] = self.update_certificate_policy()

                update_tags, self.tags = self.update_tags(response['properties']['tags'])
                if update_tags or (self.enabled is not None and bool(self.enabled) != bool(response['properties']['attributes']['enabled'])):
                    changed = True
                    if not self.check_mode:
                        response = self.update_certificate_properties()

                if self.state == 'import' and self.cert_data:
                    existing_thumb_b64 = response['properties'].get('x509_thumbprint')
                    existing_thumb = None
                    if existing_thumb_b64:
                        try:
                            existing_thumb = base64.b64decode(existing_thumb_b64)
                        except Exception:
                            existing_thumb = None

                    new_thumb = _cert_data_thumbprint_sha1(self.cert_data, (self.policy or {}).get('content_type'), password=self.password)

                    should_import = False
                    if new_thumb is None or existing_thumb is None:
                        should_import = True
                    else:
                        should_import = (new_thumb != existing_thumb)

                    if should_import:
                        changed = True
                        if not self.check_mode:
                            response = self.import_certificate()
            else:
                changed = True
                if self.state == 'import':
                    if not self.check_mode:
                        response = self.import_certificate()
                else:
                    if not self.check_mode:
                        response = self.create_certificate()

        self.results['changed'] = changed
        self.results['certificate'] = response

        return self.results

    def get_keyvault_client(self):

        return CertificateClient(vault_url=self.vault_uri, credential=self.azure_auth.azure_credential_track2)

    def get_certificate(self):
        '''
        Gets the certificate fact of the specified in key vault.

        :return: deserialized certificate state dictionary
        '''
        self.log("Get the certificate {0}".format(self.name))

        try:
            return certificatebundle_to_dict(self._client.get_certificate(certificate_name=self.name))

        except Exception as ec:
            self.log("Did not find the key vault certificate {0}: {1}".format(self.name, str(ec)))

    def create_certificate(self):
        '''
        Create the certificate in key vault.

        :return: deserialized certificate state dictionary
        '''
        self.log("Create the certificate {0}".format(self.name))

        lifetime_actions = []
        for item in (self.policy.get('lifetime_actions') or []):
            lifetime_actions.append(LifetimeAction(**item))

        try:
            if self.policy is None:
                policy = CertificatePolicy.get_default()
            else:
                policy = CertificatePolicy(subject=self.policy.get('subject'),
                                           issuer_name=self.policy.get('issuer_name'),
                                           exportable=self.policy.get('exportable'),
                                           key_type=self.policy.get('key_type'),
                                           key_size=self.policy.get('key_size'),
                                           san_emails=self.policy.get('san_emails'),
                                           content_type=self.policy.get('content_type'),
                                           validity_in_months=self.policy.get('validity_in_months'),
                                           reuse_key=self.policy.get('reuse_key'),
                                           key_curve_name=self.policy.get('key_curve_name'),
                                           enhanced_key_usage=self.policy.get('enhanced_key_usage'),
                                           key_usage=self.policy.get('key_usage'),
                                           certificate_type=self.policy.get('certificate_type'),
                                           certificate_transparency=self.policy.get('certificate_transparency'),
                                           san_dns_names=self.policy.get('san_dns_names'),
                                           lifetime_actions=lifetime_actions,
                                           san_user_principal_names=self.policy.get('san_user_principal_names'))
            response = self._client.begin_create_certificate(certificate_name=self.name,
                                                             policy=policy,
                                                             enabled=self.enabled,
                                                             tags=self.tags)
            if isinstance(response, LROPoller):
                return certificatebundle_to_dict(self.get_poller_result(response))

        except Exception as ec:
            self.fail("Did not create the key vault certificate {0}: {1}".format(self.name, str(ec)))

    def import_certificate(self):
        '''
        Import the certificate in key vault.

        :return: deserialized certificate state dictionary
        '''
        self.log("Import the certificate {0}".format(self.name))

        try:
            raw_bytes = None
            ct = (self.policy or {}).get('content_type')
            if ct and ct.lower() in ("application/x-pkcs12", "application/x-pfx"):
                # Expect cert_data to be Base64 text of the PFX; decode to raw bytes
                try:
                    raw_bytes = base64.b64decode(self.cert_data)
                except Exception:
                    # raw binary encoded as latin-1-safe text; avoid utf-8
                    raw_bytes = self.cert_data.encode("latin-1")
            else:
                # PEM or unknown: treat as text content; SDK accepts PEM bytes
                raw_bytes = self.cert_data.encode('utf-8')

            response = self._client.import_certificate(certificate_name=self.name,
                                                       certificate_bytes=raw_bytes,
                                                       enabled=self.enabled,
                                                       password=self.password,
                                                       tags=self.tags)

            if response:
                response = certificatebundle_to_dict(response)
                return response

        except Exception as ec:
            self.fail("Did not import the key vault certificate {0}: {1}".format(self.name, str(ec)))

    def delete_certificate(self):
        '''
        Delete the certificate in key vault.

        :return: deserialized certificate state dictionary
        '''
        self.log("Delete the certificate {0}".format(self.name))

        try:
            response = self._client.begin_delete_certificate(certificate_name=self.name)
            if isinstance(response, LROPoller):
                return deleted_certificatebundle_to_dict(self.get_poller_result(response))
            self.log("Delete the certificate")
        except Exception as ec:
            self.fail("Did not delete the key vault certificate {0}: {1}".format(self.name, str(ec)))

    def get_deleted_certificate(self):
        '''
        Gets the deleted certificate facts in key vault.

        :return: deserialized certificate state dictionary
        '''
        self.log("Get the certificate {0}".format(self.name))

        try:
            return deleted_certificatebundle_to_dict(self._client.get_deleted_certificate(certificate_name=self.name))
        except Exception as ec:
            self.log("Find the key vault certificate got exception, exception as {0}".format(str(ec)))

    def recover_certificate(self):
        '''
        Recover the certificate in key vault.

        :return: deserialized certificate state dictionary
        '''
        self.log("Recover the certificate {0}".format(self.name))

        try:
            response = self._client.begin_recover_deleted_certificate(certificate_name=self.name)
            if isinstance(response, LROPoller):
                return certificatebundle_to_dict(self.get_poller_result(response))

        except Exception as ec:
            self.fail("Did not recover the key vault certificate {0}: {1}".format(self.name, str(ec)))

    def update_certificate_properties(self):
        '''
        Update the certificate properties in key vault.

        :return: deserialized certificate state dictionary
        '''
        self.log("Merge the certificate {0}".format(self.name))

        try:
            response = self._client.update_certificate_properties(certificate_name=self.name,
                                                                  version=None,
                                                                  enabled=self.enabled,
                                                                  tags=self.tags)
            if response is not None:
                return certificatebundle_to_dict(response)
        except Exception as ec:
            self.fail("Did not update the key vault certificate {0}: {1}".format(self.name, str(ec)))

    def update_certificate_policy(self):
        '''
        Update the certificate policy in key vault.

        :return: deserialized certificate state dictionary
        '''
        self.log("Update the certificate policy {0}".format(self.name))

        lifetime_actions = []
        for item in (self.policy.get('lifetime_actions') or []):
            lifetime_actions.append(LifetimeAction(**item))

        policy = CertificatePolicy(subject=self.policy.get('subject'),
                                   issuer_name=self.policy.get('issuer_name'),
                                   exportable=self.policy.get('exportable'),
                                   key_type=self.policy.get('key_type'),
                                   key_size=self.policy.get('key_size'),
                                   san_emails=self.policy.get('san_emails'),
                                   content_type=self.policy.get('content_type'),
                                   validity_in_months=self.policy.get('validity_in_months'),
                                   reuse_key=self.policy.get('reuse_key'),
                                   key_curve_name=self.policy.get('key_curve_name'),
                                   enhanced_key_usage=self.policy.get('enhanced_key_usage'),
                                   key_usage=self.policy.get('key_usage'),
                                   certificate_type=self.policy.get('certificate_type'),
                                   certificate_transparency=self.policy.get('certificate_transparency'),
                                   san_dns_names=self.policy.get('san_dns_names'),
                                   lifetime_actions=lifetime_actions,
                                   san_user_principal_names=self.policy.get('san_user_principal_names'))
        try:
            response = self._client.update_certificate_policy(certificate_name=self.name,
                                                              policy=policy)
            if response is not None:
                return policy_bundle_to_dict(response)

        except Exception as ec:
            self.fail("Did not update policy in the key vault certificate {0}: {1}".format(self.name, str(ec)))

    def purge_certificate(self):
        '''
        Permanently deletes a deleted certificate.
        '''
        self.log("Permanently deletes the certificate {0}".format(self.name))

        try:
            self._client.purge_deleted_certificate(certificate_name=self.name)

        except Exception as ec:
            self.fail("Did not permanently delete the key vault certificate {0}: {1}".format(self.name, str(ec)))


def main():
    """Main execution"""
    AzureRMKeyVaultCertificate()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_passcode_profile
short_description: Manages passcode-profile objects on Checkpoint over Web Services API
description:
  - Manages passcode-profile objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82 management version.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  allow_simple_passcode:
    description:
      - The passcode length is 4 and only numeric values allowed.
    type: bool
  min_passcode_length:
    description:
      - Minimum passcode length - relevant if "allow-simple-passcode" is disable.
    type: int
  require_alphanumeric_passcode:
    description:
      - Require alphanumeric characters in the passcode - relevant if "allow-simple-passcode" is disable.
    type: bool
  min_passcode_complex_characters:
    description:
      - Minimum number of complex characters (if "require-alphanumeric-passcode" is enabled). The number of the complex characters cannot be greater
        than number of the passcode length.
    type: int
  force_passcode_expiration:
    description:
      - Enable/disable expiration date to the passcode.
    type: bool
  passcode_expiration_period:
    description:
      - The period in days after which the passcode will expire.
    type: int
  enable_inactivity_time_lock:
    description:
      - Lock the device if app is inactive.
    type: bool
  max_inactivity_time_lock:
    description:
      - Time without user input before passcode must be re-entered (in minutes).
    type: int
  enable_passcode_failed_attempts:
    description:
      - Exit after few failures in passcode verification.
    type: bool
  max_passcode_failed_attempts:
    description:
      - Number of failed attempts allowed.
    type: int
  enable_passcode_history:
    description:
      - Check passcode history for reparations.
    type: bool
  passcode_history:
    description:
      - Number of passcodes that will be kept in history.
    type: int
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-passcode-profile
  cp_mgmt_passcode_profile:
    name: New App Passcode Policy
    state: present

- name: set-passcode-profile
  cp_mgmt_passcode_profile:
    allow_simple_passcode: 'true'
    max_inactivity_time_lock: '30'
    name: New App Passcode Policy
    require_alphanumeric_passcode: 'false'
    state: present

- name: delete-passcode-profile
  cp_mgmt_passcode_profile:
    name: My App Passcode Policy
    state: absent
"""

RETURN = """
cp_mgmt_passcode_profile:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        allow_simple_passcode=dict(type='bool'),
        min_passcode_length=dict(type='int', no_log=True),
        require_alphanumeric_passcode=dict(type='bool'),
        min_passcode_complex_characters=dict(type='int', no_log=True),
        force_passcode_expiration=dict(type='bool'),
        passcode_expiration_period=dict(type='int'),
        enable_inactivity_time_lock=dict(type='bool'),
        max_inactivity_time_lock=dict(type='int'),
        enable_passcode_failed_attempts=dict(type='bool'),
        max_passcode_failed_attempts=dict(type='int', no_log=True),
        enable_passcode_history=dict(type='bool'),
        passcode_history=dict(type='int', no_log=True),
        tags=dict(type='list', elements='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'passcode-profile'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2014, Michael J. Schultz <mjschultz@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: sns
short_description: Send Amazon Simple Notification Service messages
version_added: 1.0.0
description:
  - Sends a notification to a topic on your Amazon SNS account.
author:
  - Michael J. Schultz (@mjschultz)
  - Paul Arthur (@flowerysong)
options:
  msg:
    description:
      - Default message for subscriptions without a more specific message.
    required: true
    aliases: [ "default" ]
    type: str
  subject:
    description:
      - Message subject
    type: str
  topic:
    description:
      - The name or ARN of the topic to publish to.
    required: true
    type: str
  email:
    description:
      - Message to send to email subscriptions.
    type: str
  email_json:
    description:
      - Message to send to email-json subscriptions.
    type: str
  sqs:
    description:
      - Message to send to SQS subscriptions.
    type: str
  sms:
    description:
      - Message to send to SMS subscriptions.
    type: str
  http:
    description:
      - Message to send to HTTP subscriptions.
    type: str
  https:
    description:
      - Message to send to HTTPS subscriptions.
    type: str
  application:
    description:
      - Message to send to application subscriptions.
    type: str
  lambda:
    description:
      - Message to send to Lambda subscriptions.
    type: str
  message_attributes:
    description:
      - Dictionary of message attributes. These are optional structured data entries to be sent along to the endpoint.
      - This is in AWS's distinct Name/Type/Value format; see example below.
    type: dict
  message_structure:
    description:
      - The payload format to use for the message.
      - This must be C(json) to support protocol-specific messages (C(http), C(https), C(email), C(sms), C(sqs)).
      - It must be C(string) to support I(message_attributes).
    default: json
    choices: ['json', 'string']
    type: str
  message_group_id:
    description:
      - A tag which is used to process messages that belong to the same group in a FIFO manner.
      - Has to be included when publishing a message to a fifo topic.
      - Can contain up to 128 alphanumeric characters and punctuation.
    type: str
    version_added: 5.4.0
  message_deduplication_id:
    description:
      - Only in connection with the message_group_id.
      - Overwrites the auto generated MessageDeduplicationId.
      - Can contain up to 128 alphanumeric characters and punctuation.
      - Messages with the same deduplication id getting recognized as the same message.
      - Gets overwritten by an auto generated token, if the topic has ContentBasedDeduplication set.
    type: str
    version_added: 5.4.0

extends_documentation_fragment:
  - amazon.aws.region.modules
  - amazon.aws.common.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Send default notification message via SNS
  community.aws.sns:
    msg: '{{ inventory_hostname }} has completed the play.'
    subject: Deploy complete!
    topic: deploy
  delegate_to: localhost

- name: Send notification messages via SNS with short message for SMS
  community.aws.sns:
    msg: '{{ inventory_hostname }} has completed the play.'
    sms: deployed!
    subject: Deploy complete!
    topic: deploy
  delegate_to: localhost

- name: Send message with message_attributes
  community.aws.sns:
    topic: "deploy"
    msg: "message with extra details!"
    message_attributes:
      channel:
        data_type: String
        string_value: "mychannel"
      color:
        data_type: String
        string_value: "green"
  delegate_to: localhost

- name: Send message to a fifo topic
  community.aws.sns:
    topic: "deploy"
    msg: "Message with message group id"
    subject: Deploy complete!
    message_group_id: "deploy-1"
  delegate_to: localhost
"""

RETURN = r"""
msg:
  description: Human-readable diagnostic information
  returned: always
  type: str
  sample: OK
message_id:
  description: The message ID of the submitted message
  returned: when success
  type: str
  sample: 2f681ef0-6d76-5c94-99b2-4ae3996ce57b
sequence_number:
  description: A 128 bits long sequence number which gets assigned to the message in fifo topics
  returned: when success
  type: str
"""

import json

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.sns import topic_arn_lookup


def main():
    protocols = [
        "http",
        "https",
        "email",
        "email_json",
        "sms",
        "sqs",
        "application",
        "lambda",
    ]

    argument_spec = dict(
        msg=dict(required=True, aliases=["default"]),
        subject=dict(),
        topic=dict(required=True),
        message_attributes=dict(type="dict"),
        message_structure=dict(choices=["json", "string"], default="json"),
        message_group_id=dict(),
        message_deduplication_id=dict(),
    )

    for p in protocols:
        argument_spec[p] = dict()

    module = AnsibleAWSModule(argument_spec=argument_spec)

    sns_kwargs = dict(
        Message=module.params["msg"],
        Subject=module.params["subject"],
        MessageStructure=module.params["message_structure"],
    )

    if module.params["message_attributes"]:
        if module.params["message_structure"] != "string":
            module.fail_json(msg='message_attributes is only supported when the message_structure is "string".')
        sns_kwargs["MessageAttributes"] = module.params["message_attributes"]

    if module.params["message_group_id"]:
        sns_kwargs["MessageGroupId"] = module.params["message_group_id"]
        if module.params["message_deduplication_id"]:
            sns_kwargs["MessageDeduplicationId"] = module.params["message_deduplication_id"]

    dict_msg = {"default": sns_kwargs["Message"]}

    for p in protocols:
        if module.params[p]:
            if sns_kwargs["MessageStructure"] != "json":
                module.fail_json(msg='Protocol-specific messages are only supported when message_structure is "json".')
            dict_msg[p.replace("_", "-")] = module.params[p]

    client = module.client("sns")

    topic = module.params["topic"]
    if ":" in topic:
        # Short names can't contain ':' so we'll assume this is the full ARN
        sns_kwargs["TopicArn"] = topic
    else:
        sns_kwargs["TopicArn"] = topic_arn_lookup(client, module, topic)

    if not sns_kwargs["TopicArn"]:
        module.fail_json(msg=f"Could not find topic: {topic}")

    if sns_kwargs["MessageStructure"] == "json":
        sns_kwargs["Message"] = json.dumps(dict_msg)

    try:
        result = client.publish(**sns_kwargs)
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg="Failed to publish message")

    sns_result = dict(msg="OK", message_id=result["MessageId"])

    if module.params["message_group_id"]:
        sns_result["sequence_number"] = result["SequenceNumber"]

    module.exit_json(**sns_result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2018, Dag Wieers (dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: cobbler_system
short_description: Manage system objects in Cobbler
description:
  - Add, modify or remove systems in Cobbler.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
options:
  host:
    description:
      - The name or IP address of the Cobbler system.
    default: 127.0.0.1
    type: str
  port:
    description:
      - Port number to be used for REST connection.
      - The default value depends on parameter O(use_ssl).
    type: int
  username:
    description:
      - The username to log in to Cobbler.
    default: cobbler
    type: str
  password:
    description:
      - The password to log in to Cobbler.
    type: str
  use_ssl:
    description:
      - If V(false), an HTTP connection is used instead of the default HTTPS connection.
    type: bool
    default: true
  validate_certs:
    description:
      - If V(false), SSL certificates are not validated.
      - This should only set to V(false) when used on personally controlled sites using self-signed certificates.
    type: bool
    default: true
  name:
    description:
      - The system name to manage.
    type: str
  properties:
    description:
      - A dictionary with system properties.
    type: dict
  interfaces:
    description:
      - A list of dictionaries containing interface options.
    type: dict
  sync:
    description:
      - Sync on changes.
      - Concurrently syncing Cobbler is bound to fail.
    type: bool
    default: false
  state:
    description:
      - Whether the system should be present, absent or a query is made.
    choices: [absent, present, query]
    default: present
    type: str
author:
  - Dag Wieers (@dagwieers)
notes:
  - Concurrently syncing Cobbler is bound to fail with weird errors.
"""

EXAMPLES = r"""
- name: Ensure the system exists in Cobbler
  community.general.cobbler_system:
    host: cobbler01
    username: cobbler
    password: MySuperSecureP4sswOrd
    name: myhost
    properties:
      profile: CentOS6-x86_64
      name_servers: [2.3.4.5, 3.4.5.6]
      name_servers_search: foo.com, bar.com
    interfaces:
      eth0:
        macaddress: 00:01:02:03:04:05
        ipaddress: 1.2.3.4
  delegate_to: localhost

- name: Enable network boot in Cobbler
  community.general.cobbler_system:
    host: bdsol-aci-cobbler-01
    username: cobbler
    password: ins3965!
    name: bdsol-aci51-apic1.cisco.com
    properties:
      netboot_enabled: true
    state: present
  delegate_to: localhost

- name: Query all systems in Cobbler
  community.general.cobbler_system:
    host: cobbler01
    username: cobbler
    password: MySuperSecureP4sswOrd
    state: query
  register: cobbler_systems
  delegate_to: localhost

- name: Query a specific system in Cobbler
  community.general.cobbler_system:
    host: cobbler01
    username: cobbler
    password: MySuperSecureP4sswOrd
    name: '{{ inventory_hostname }}'
    state: query
  register: cobbler_properties
  delegate_to: localhost

- name: Ensure the system does not exist in Cobbler
  community.general.cobbler_system:
    host: cobbler01
    username: cobbler
    password: MySuperSecureP4sswOrd
    name: myhost
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
systems:
  description: List of systems.
  returned: O(state=query) and O(name) is not provided
  type: list
system:
  description: (Resulting) information about the system we are working with.
  returned: when O(name) is provided
  type: dict
"""

import ssl
import xmlrpc.client as xmlrpc_client

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)

IFPROPS_MAPPING = dict(
    bondingopts="bonding_opts",
    bridgeopts="bridge_opts",
    connected_mode="connected_mode",
    cnames="cnames",
    dhcptag="dhcp_tag",
    dnsname="dns_name",
    ifgateway="if_gateway",
    interfacetype="interface_type",
    interfacemaster="interface_master",
    ipaddress="ip_address",
    ipv6address="ipv6_address",
    ipv6defaultgateway="ipv6_default_gateway",
    ipv6mtu="ipv6_mtu",
    ipv6prefix="ipv6_prefix",
    ipv6secondaries="ipv6_secondariesu",
    ipv6staticroutes="ipv6_static_routes",
    macaddress="mac_address",
    management="management",
    mtu="mtu",
    netmask="netmask",
    static="static",
    staticroutes="static_routes",
    virtbridge="virt_bridge",
)


def getsystem(conn, name, token):
    system = dict()
    if name:
        # system = conn.get_system(name, token)
        systems = conn.find_system(dict(name=name), token)
        if systems:
            system = systems[0]
    return system


def main():
    module = AnsibleModule(
        argument_spec=dict(
            host=dict(type="str", default="127.0.0.1"),
            port=dict(type="int"),
            username=dict(type="str", default="cobbler"),
            password=dict(type="str", no_log=True),
            use_ssl=dict(type="bool", default=True),
            validate_certs=dict(type="bool", default=True),
            name=dict(type="str"),
            interfaces=dict(type="dict"),
            properties=dict(type="dict"),
            sync=dict(type="bool", default=False),
            state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        ),
        supports_check_mode=True,
    )

    username = module.params["username"]
    password = module.params["password"]
    port = module.params["port"]
    use_ssl = module.params["use_ssl"]
    validate_certs = module.params["validate_certs"]

    name = module.params["name"]
    state = module.params["state"]

    proto = "https" if use_ssl else "http"
    if not port:
        port = "443" if use_ssl else "80"

    result = dict(
        changed=False,
    )

    start = now()

    ssl_context = None if validate_certs or not use_ssl else ssl._create_unverified_context()

    url = f"{proto}://{module.params['host']}:{port}/cobbler_api"
    conn = xmlrpc_client.ServerProxy(url, context=ssl_context)

    try:
        token = conn.login(username, password)
    except xmlrpc_client.Fault as e:
        module.fail_json(msg=f"Failed to log in to Cobbler '{url}' as '{username}'. {e}")
    except Exception as e:
        module.fail_json(msg=f"Connection to '{url}' failed. {e}")

    system = getsystem(conn, name, token)
    # result['system'] = system

    if state == "query":
        if name:
            result["system"] = system
        else:
            # Turn it into a dictionary of dictionaries
            # all_systems = conn.get_systems()
            # result['systems'] = { system['name']: system for system in all_systems }

            # Return a list of dictionaries
            result["systems"] = conn.get_systems()

    elif state == "present":
        if system:
            # Update existing entry
            system_id = ""
            # https://github.com/cobbler/cobbler/blame/v3.3.7/cobbler/api.py#L277
            if float(conn.version()) >= 3.4:
                system_id = conn.get_system_handle(name)
            else:
                system_id = conn.get_system_handle(name, token)

            for key, value in module.params["properties"].items():
                if key not in system:
                    module.warn(f"Property '{key}' is not a valid system property.")
                if system[key] != value:
                    try:
                        conn.modify_system(system_id, key, value, token)
                        result["changed"] = True
                    except Exception as e:
                        module.fail_json(msg=f"Unable to change '{key}' to '{value}'. {e}")

        else:
            # Create a new entry
            system_id = conn.new_system(token)
            conn.modify_system(system_id, "name", name, token)
            result["changed"] = True

            if module.params["properties"]:
                for key, value in module.params["properties"].items():
                    try:
                        conn.modify_system(system_id, key, value, token)
                    except Exception as e:
                        module.fail_json(msg=f"Unable to change '{key}' to '{value}'. {e}")

        # Add interface properties
        interface_properties = dict()
        if module.params["interfaces"]:
            for device, values in module.params["interfaces"].items():
                for key, value in values.items():
                    if key == "name":
                        continue
                    if key not in IFPROPS_MAPPING:
                        module.warn(f"Property '{key}' is not a valid system property.")
                        continue
                    if not system or system["interfaces"].get(device, {}).get(IFPROPS_MAPPING[key]) != value:
                        result["changed"] = True
                    interface_properties[f"{key}-{device}"] = value

            if result["changed"] is True:
                conn.modify_system(system_id, "modify_interface", interface_properties, token)

        # Only save when the entry was changed
        if not module.check_mode and result["changed"]:
            conn.save_system(system_id, token)

    elif state == "absent":
        if system:
            if not module.check_mode:
                conn.remove_system(name, token)
            result["changed"] = True

    if not module.check_mode and module.params["sync"] and result["changed"]:
        try:
            conn.sync(token)
        except Exception as e:
            module.fail_json(msg=f"Failed to sync Cobbler. {e}")

    if state in ("absent", "present"):
        result["system"] = getsystem(conn, name, token)

        if module._diff:
            result["diff"] = dict(before=system, after=result["system"])

    elapsed = now() - start
    module.exit_json(elapsed=elapsed.seconds, **result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
"""
The module file for ios_lldp_interfaces
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: ios_lldp_interfaces
short_description: Resource module to configure LLDP interfaces.
description:
  This module manages link layer discovery protocol (LLDP) attributes of
  interfaces on Cisco IOS devices.
version_added: 1.0.0
author: Sumit Jaiswal (@justjais)
notes:
  - Tested against Cisco IOSXE Version 17.3 on CML.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  config:
    description: A dictionary of LLDP options
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Full name of the interface excluding any logical unit number, i.e. GigabitEthernet0/1.
        type: str
        required: true
      receive:
        description:
          - Enable LLDP reception on interface.
        type: bool
      transmit:
        description:
          - Enable LLDP transmission on interface.
        type: bool
      med_tlv_select:
        description:
          - Selection of LLDP MED TLVs to send
          - NOTE, if med-tlv-select is configured idempotency won't be maintained as
            Cisco device doesn't record configured med-tlv-select options. As such,
            Ansible cannot verify if the respective med-tlv-select options is already
            configured or not from the device side. If you try to apply med-tlv-select
            option in every play run, Ansible will show changed as True.
        type: dict
        suboptions:
          inventory_management:
            description:
              - LLDP MED Inventory Management TLV
            type: bool
      tlv_select:
        description:
          - Selection of LLDP type-length-value i.e. TLVs to send
          - NOTE, if tlv-select is configured idempotency won't be maintained as Cisco
            device doesn't record configured tlv-select options. As such, Ansible cannot
            verify if the respective tlv-select options is already configured or not
            from the device side. If you try to apply tlv-select option in every play
            run, Ansible will show changed as True.
        type: dict
        suboptions:
          power_management:
            description:
              - IEEE 802.3 DTE Power via MDI TLV
            type: bool
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(sh lldp interface).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config
        | include ip route|ipv6 route) executed on device. For state I(parsed) active
        connection to remote host is not required.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - parsed
    default: merged
"""

EXAMPLES = """
# Using merged
#
# Before state:
# -------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#

- name: Merge provided configuration with device configuration
  cisco.ios.ios_lldp_interfaces:
    config:
      - name: GigabitEthernet0/1
        receive: true
        transmit: true
      - name: GigabitEthernet0/2
        receive: true
      - name: GigabitEthernet0/3
        transmit: true
    state: merged

# After state:
# ------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: disabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: enabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#

# Using overridden
#
# Before state:
# -------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME

- name: Override device configuration of all lldp_interfaces with provided configuration
  cisco.ios.ios_lldp_interfaces:
    config:
      - name: GigabitEthernet0/2
        receive: true
        transmit: true
    state: overridden

# After state:
# ------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME

# Using replaced
#
# Before state:
# -------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#

- name: Replaces device configuration of listed lldp_interfaces with provided configuration
  cisco.ios.ios_lldp_interfaces:
    config:
      - name: GigabitEthernet0/2
        receive: true
        transmit: true
      - name: GigabitEthernet0/3
        receive: true
    state: replaced

# After state:
# ------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: disabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#

# Using Deleted
#
# Before state:
# -------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME

- name: "Delete LLDP attributes of given interfaces (Note: This won't delete the interface itself)"
  cisco.ios.ios_lldp_interfaces:
    config:
      - name: GigabitEthernet0/1
    state: deleted

# After state:
# -------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/2:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#

# Using Deleted without any config passed
# "(NOTE: This will delete all of configured LLDP module attributes)"
#
# Before state:
# -------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME

- name: "Delete LLDP attributes for all configured interfaces (Note: This won't delete the interface itself)"
  cisco.ios.ios_lldp_interfaces:
    state: deleted

# After state:
# -------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: INIT
#
# GigabitEthernet0/3:
#    Tx: disabled
#    Rx: disabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME

# Using Gathered

# Before state:
# -------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME

- name: Gather listed LLDP interfaces with provided configurations
  cisco.ios.ios_lldp_interfaces:
    config:
    state: gathered

# Module Execution Result:
# ------------------------
#
# "gathered": [
#         {
#             "name": "GigabitEthernet0/0",
#             "receive": true,
#             "transmit": true
#         },
#         {
#             "name": "GigabitEthernet0/1",
#             "receive": true,
#             "transmit": true
#         },
#         {
#             "name": "GigabitEthernet0/2",
#             "receive": true,
#             "transmit": true
#         }
#     ]

# After state:
# ------------
#
# vios#sh lldp interface
# GigabitEthernet0/0:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME

# GigabitEthernet0/2:
#    Tx: enabled
#    Rx: enabled
#    Tx state: IDLE
#    Rx state: WAIT FOR FRAME

# Using Rendered

- name: Render the commands for provided  configuration
  cisco.ios.ios_lldp_interfaces:
    config:
      - name: GigabitEthernet0/0
        receive: true
        transmit: true
      - name: GigabitEthernet0/1
        receive: true
        transmit: true
      - name: GigabitEthernet0/2
        receive: true
    state: rendered

# Module Execution Result:
# ------------------------
#
# "rendered": [
#         "interface GigabitEthernet0/0",
#         "lldp receive",
#         "lldp transmit",
#         "interface GigabitEthernet0/1",
#         "lldp receive",
#         "lldp transmit",
#         "interface GigabitEthernet0/2",
#         "lldp receive"
#     ]

# Using Parsed

# File: parsed.cfg
# ----------------
#
# GigabitEthernet0/0:
#   Tx: enabled
#   Rx: disabled
#   Tx state: IDLE
#   Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/1:
#   Tx: enabled
#   Rx: enabled
#   Tx state: IDLE
#   Rx state: WAIT FOR FRAME
#
# GigabitEthernet0/2:
#   Tx: disabled
#   Rx: enabled
#   Tx state: IDLE
#   Rx state: INIT

- name: Parse the commands for provided configuration
  cisco.ios.ios_lldp_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Module Execution Result:
# ------------------------
#
# "parsed": [
#         {
#             "name": "GigabitEthernet0/0",
#             "receive": false,
#             "transmit": true
#         },
#         {
#             "name": "GigabitEthernet0/1",
#             "receive": true,
#             "transmit": true
#         },
#         {
#             "name": "GigabitEthernet0/2",
#             "receive": true,
#             "transmit": false
#         }
#     ]
"""

RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['interface GigabitEthernet 0/1', 'lldp transmit', 'lldp receive']
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.lldp_interfaces.lldp_interfaces import (
    Lldp_InterfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.lldp_interfaces.lldp_interfaces import (
    Lldp_Interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]

    module = AnsibleModule(
        argument_spec=Lldp_InterfacesArgs.argument_spec,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )
    result = Lldp_Interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The remove_keys plugin code
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible.errors import AnsibleFilterError


def _raise_error(msg):
    """Raise an error message, prepend with filter name
    :param msg: The message
    :type msg: str
    :raises: AnsibleError
    """
    error = "Error when using plugin 'remove_keys': {msg}".format(msg=msg)
    raise AnsibleFilterError(error)


def remove_keys_from_dict_n_list(data, target, matching_parameter):
    if isinstance(data, dict):
        for key in set(target):
            for k in list(data.keys()):
                if matching_parameter == "regex":
                    if re.match(key, k):
                        del data[k]
                elif matching_parameter == "starts_with":
                    if k.startswith(key):
                        del data[k]
                elif matching_parameter == "ends_with":
                    if k.endswith(key):
                        del data[k]
                else:
                    if k == key:
                        del data[k]
        for k, v in data.items():
            remove_keys_from_dict_n_list(v, target, matching_parameter)
    elif isinstance(data, list):
        for i in data:
            remove_keys_from_dict_n_list(i, target, matching_parameter)
    return data


def clear_empty_data(data):
    if isinstance(data, dict):
        # for k in list(data.keys()):
        #     if not data.get(k, {}):
        #         del data[k]
        for k, v in data.items():
            data[k] = clear_empty_data(v)
    if isinstance(data, list):
        temp = []
        for i in data:
            if i:
                temp.append(clear_empty_data(i))
        return temp
    return data


def remove_keys(data, target, matching_parameter="equality"):
    """Remove unwanted keys recursively from a given data"
    :param data: The data passed in (data|remove_keys(...))
    :type data: raw
    :param target: List of keys on with operation is to be performed
    :type data: list
    :type elements: string
    :param matching_parameter: matching type of the target keys with data keys
    :type data: str
    """
    if not isinstance(data, (list, dict)):
        _raise_error("Input is not valid for attribute removal")
    data = remove_keys_from_dict_n_list(data, target, matching_parameter)
    data = clear_empty_data(data)
    return data



#!/usr/bin/python
#
# Copyright (c) 2020 Fred-Sun, (@Fred-Sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualwan_info
version_added: '1.5.0'
short_description: Get VirtualWan info
description:
    - Get info of VirtualWan.
options:
    resource_group:
        description:
            - The resource group name of the VirtualWan.
        type: str
    name:
        description:
            - The name of the VirtualWAN being retrieved.
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Fred-Sun (@Fred-Sun)
'''

EXAMPLES = '''
- name: Get Virtual WAN by name
  azure_rm_virtualwan_info:
    resource_group: myResouceGroup
    name: testwan

- name: List all Virtual WANLs by resource group
  azure_rm_virtualwan_info:
    resource_group: myResourceGroup

- name: List all Virtual WANs by subscription_id
  azure_rm_virtualwan_info:
'''

RETURN = '''
virtual_wans:
    description:
        - A list of dict results where the key is the name of the VirtualWan and the values are the facts for that VirtualWan.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualWans/testwan
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: testwan
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample: Microsoft.Network/virtualWans
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { 'key1': 'value1'}
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            returned: always
            type: str
            sample:  "86df6f3d-19f2-4cc8-8574-47921de4a6f1"
        disable_vpn_encryption:
            description:
                - Vpn encryption to be disabled or not.
            returned: always
            type: bool
            sample: false
        virtual_hubs:
            description:
                - List of VirtualHubs in the VirtualWAN.
            type: complex
            contains:
                id:
                    description:
                        - The virtual hubs list of the virtual wan.
                    returned: always
                    type: str
                    sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualHubs/test
        vpn_sites:
            description:
                - List of VpnSites in the VirtualWAN.
            returned: always
            type: complex
            contains:
                id:
                    description:
                        - The vpn site resource ID.
                    returned: always
                    type: str
                    sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/vpnSites/test1
        allow_branch_to_branch_traffic:
            description:
                - True if branch to branch traffic is allowed.
            returned: always
            type: bool
            sample: true
        allow_vnet_to_vnet_traffic:
            description:
                - True if Vnet to Vnet traffic is allowed.
            returned: always
            type: bool
            sample: True
        office365_local_breakout_category:
            description:
                - The office local breakout category.
            returned: always
            type: str
            sample: None
        provisioning_state:
            description:
                - The provisioning state of the virtual WAN resource.
            returned: always
            type: str
            sample: Succeeded
        virtual_wan_type:
            description:
                - The type of virtual wan.
            returned: always
            type: str
            sample: Standard
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase
try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMVirtualWanInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str'
            ),
            name=dict(
                type='str'
            )
        )

        self.resource_group = None
        self.name = None

        self.results = dict(changed=False)

        super(AzureRMVirtualWanInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if (self.resource_group is not None and self.name is not None):
            self.results['virtual_wans'] = self.format_item(self.get())
        elif (self.resource_group is not None):
            self.results['virtual_wans'] = self.format_item(self.list_by_resource_group())
        else:
            self.results['virtual_wans'] = self.format_item(self.list())
        return self.results

    def get(self):
        response = None

        try:
            response = self.network_client.virtual_wans.get(resource_group_name=self.resource_group,
                                                            virtual_wan_name=self.name)
        except ResourceNotFoundError as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def list_by_resource_group(self):
        response = None

        try:
            response = self.network_client.virtual_wans.list_by_resource_group(resource_group_name=self.resource_group)
        except ResourceNotFoundError as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def list(self):
        response = None

        try:
            response = self.network_client.virtual_wans.list()
        except ResourceNotFoundError as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def format_item(self, item):
        if hasattr(item, 'as_dict'):
            return [item.as_dict()]
        else:
            result = []
            items = list(item)
            for tmp in items:
                result.append(tmp.as_dict())
            return result


def main():
    AzureRMVirtualWanInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_eventgrid_subscription_info
version_added: "3.13.0"
short_description: List Eventgrid Event Subscriptions
description:
    - List Eventgrid Event Subscriptions.
options:
    source_resource_id:
        description:
            - Fully qualified identifier of the Azure resource whose event subscription
              needs to be listed.
        type: str
        required: false
    location:
        description:
            - List by region.
        type: str
        required: false
    resource_group:
        description:
            - Name of a resource group to limit event subscriptions from.  If not specified the subscription
              will be searched.
        required: false
        type: str
    odata_query:
        description:
            - The query used to filter the search results using OData syntax.
        required: false
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Get info on an event subscription
  azure.azcollection.azure_rm_eventgrid_subscription_info:
    location: westus3
    odata_query: "contains(name, 'PATTERN')"
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
eventgrid_subscriptions:
    description:
        - List of Eventgrid subscriptions.
    returned: always
    type: complex
    contains:
        destination:
            description:
                - Details of the destination
            returned: always
            type: complex
            contains:
                delivery_attribute_mappings:
                    description:
                        - Mapping attributes for delivery
                    returned: always
                    type: str
                endpoint_base_url:
                    description:
                        - The base URL of the endpoint
                    returned: always
                    type: str
                    sample: https://event-grid-site-xxxxx20070.azurewebsites.net/api/updates
                endpoint_type:
                    description:
                        - The type of the endpoint
                    returned: always
                    type: str
                    sample: WebHook
                endpoint_url:
                    description:
                        - The URL of the endpoint
                    returned: always
                    type: str
                    sample: https://event-grid-site-xxxxx20070.azurewebsites.net/api/updates
                max_events_per_batch:
                    description:
                        - How many events will be batched
                    returned: always
                    type: int
                    sample: 1
                preferred_batch_size_in_kilobytes:
                    description:
                        - The preferred batch size in kilobytes
                    returned: always
                    type: int
                    sample: 64
        event_delivery_schema:
            description:
                - The type of Eventgrid Schema
            returned: always
            type: str
            sample: EventGridSchema
        filter:
            description:
                - Filter settings for which events to publish to endpoint
            returned: always
            type: complex
            contains:
                subject_begins_with:
                    description:
                        - Filter on Subject beginning with this value
                    returned: always
                    type: str
                subject_ends_with:
                    description:
                        - Filter on Subject ending with this value
                    returned: always
                    type: str
        id:
            description:
                - The ID of the Eventgrid Subscription
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/MyResourceGroup/providers/Microsoft.EventGrid/topics/topic-xxxxx20070/providers/Microsoft.EventGrid/eventSubscriptions/subscription-xxxxx20070 # nopep8: E501
        name:
            description:
                - The name of the Eventgrid Subscription
            returned: always
            type: str
            sample: subscription-xxxxx20070
        provisioning_state:
            description:
                - The provisioning state of this Eventgrid Subscription
            returned: always
            type: str
            sample: Succeeded
        retry_policy:
            description:
                - Retry policy for this Eventgrid Subscription
            returned: always
            type: complex
            contains:
                event_time_to_live_in_minutes:
                    description:
                        - Event time to live in minutes
                    returned: always
                    type: int
                    sample: 1440
                max_delivery_attempts:
                    description:
                        - Max attempts to deliver to endpoint
                    returned: always
                    type: int
                    sample: 30
        topic:
            description:
                - The source resource id that the Eventgrid Subscription is subscribed to
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/MyResourceGroup/providers/microsoft.eventgrid/topics/topic-xxxxx20070
        type:
            description:
                - The type of Eventgrid Subscription
            returned: always
            type: str
            sample: Microsoft.EventGrid/eventSubscriptions
'''

DEFAULT_TOP = 100
EVENTGRID_NAMESPACE = "Microsoft.EventGrid"
EVENTGRID_DOMAINS = "domains"
EVENTGRID_TOPICS = "topics"

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.mgmt.core.tools import parse_resource_id
    from azure.mgmt.eventgrid import EventGridManagementClient
    from azure.mgmt.eventgrid.models import (
        WebHookEventSubscriptionDestination,
        StorageQueueEventSubscriptionDestination,
    )
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMEventgridSubscriptionInfo(AzureRMModuleBase):

    @property
    def client(self):
        self.log('Getting client')
        if not self._client:
            self._client = self.get_mgmt_svc_client(EventGridManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)
        return self._client

    def __init__(self):

        self.module_arg_spec = dict(
            source_resource_id=dict(
                type='str',
                required=False
            ),
            location=dict(
                type='str',
                required=False
            ),
            odata_query=dict(
                type='str',
                required=False
            ),
            resource_group=dict(
                type='str',
                required=False
            ),
        )

        mutually_exclusive = [('source_resource_id', 'location')]
        required_one_of = [['source_resource_id', 'location']]

        self._client = None

        self.results = dict(
            changed=False,
            eventgrid_subscriptions=[]
        )

        super(AzureRMEventgridSubscriptionInfo, self).__init__(self.module_arg_spec,
                                                               supports_tags=False,
                                                               supports_check_mode=True,
                                                               mutually_exclusive=mutually_exclusive,
                                                               required_one_of=required_one_of,
                                                               facts_module=True,
                                                               )

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.source_resource_id is not None:
            self.results['eventgrid_subscriptions'] = self.list_by_resource_id(self.source_resource_id,
                                                                               self.odata_query,
                                                                               DEFAULT_TOP)
        else:
            self.results['eventgrid_subscriptions'] = self.list_by_location(self.resource_group,
                                                                            self.location,
                                                                            self.odata_query,
                                                                            DEFAULT_TOP)

        return self.results

    def list_by_resource_id(self, source_resource_id, odata_query, top):
        self.log('List all items by resource id')
        try:
            items = self._list_event_subscriptions_by_resource_id(self.source_resource_id,
                                                                  self.odata_query,
                                                                  top)
        except ResourceNotFoundError as exc:
            self.fail("Failed to list all items in resource group - {0}".format(str(exc)))

        return self.filter_results(items)

    def list_by_location(self, resource_group_name, location, odata_query, top):
        self.log('List all items by location')
        try:
            if resource_group_name:
                items = self.client.event_subscriptions.list_regional_by_resource_group(resource_group_name,
                                                                                        location,
                                                                                        odata_query,
                                                                                        top)
            else:
                items = self.client.event_subscriptions.list_regional_by_subscription(location,
                                                                                      odata_query,
                                                                                      top)
        except ResourceNotFoundError as exc:
            self.fail("Failed to list all items - {0}".format(str(exc)))

        return self.filter_results(items)

    def filter_results(self, items):
        results = []
        for item in items:
            subscription = self.format_eventgrid_subscription(item)
            destination = item.destination
            if isinstance(destination, WebHookEventSubscriptionDestination):
                full_endpoint_url = self.client.event_subscriptions.get_full_url(item.topic, item.name)
                subscription["destination"]["endpoint_url"] = full_endpoint_url.endpoint_url

            if not isinstance(destination, StorageQueueEventSubscriptionDestination):
                delivery_attributes = self.client.event_subscriptions.get_delivery_attributes(item.topic, item.name)
                subscription["destination"]["delivery_attribute_mappings"] = delivery_attributes.as_dict()
            results.append(subscription)
        return results

    def format_eventgrid_subscription(self, event_subscription_data):
        results = event_subscription_data and event_subscription_data.as_dict() or {}
        return results

    def _list_event_subscriptions_by_resource_id(self, resource_id, oDataQuery, top):
        # parse_resource_id doesn't handle resource_ids for Azure subscriptions and RGs
        # so, first try to look for those two patterns.
        if resource_id is not None:
            id_parts = list(filter(None, resource_id.split('/')))
            if len(id_parts) < 5:
                # Azure subscriptions or Resource group
                if id_parts[0].lower() != "subscriptions":
                    self.fail('The specified value for source_resource_id is not in the'
                              ' expected format. It should start with /subscriptions.')

                subscription_id = id_parts[1]

                if len(id_parts) == 2:
                    return self.client.event_subscriptions.list_global_by_subscription_for_topic_type(
                        "Microsoft.Resources.Subscriptions",
                        oDataQuery,
                        top)

            if len(id_parts) == 4 and id_parts[2].lower() == "resourcegroups":
                resource_group_name = id_parts[3]
                if resource_group_name is None:
                    self.fail('The specified value for source_resource_id is not'
                              ' in the expected format. A valid value for'
                              ' resource group must be provided.')
                return self.client.event_subscriptions.list_global_by_resource_group_for_topic_type(
                    resource_group_name,
                    "Microsoft.Resources.ResourceGroups",
                    oDataQuery,
                    top)

        id_parts = parse_resource_id(resource_id)
        subscription_id = id_parts.get('subscription')

        rg_name = id_parts.get('resource_group')
        resource_name = id_parts.get('name')
        namespace = id_parts.get('namespace')
        resource_type = id_parts.get('type')

        if (subscription_id is None or rg_name is None or resource_name is None or
                namespace is None or resource_type is None):
            self.fail('The specified value for source_resource_id is not in the expected format.')

        # If this is for a domain topic, invoke the appropriate operation
        if (namespace.lower() == EVENTGRID_NAMESPACE.lower() and resource_type.lower() == EVENTGRID_DOMAINS.lower()):
            child_resource_type = id_parts.get('child_type_1')
            child_resource_name = id_parts.get('child_name_1')

            if (child_resource_type is not None and child_resource_type.lower() == EVENTGRID_TOPICS.lower() and
                    child_resource_name is not None):
                return self.client.event_subscriptions.list_by_domain_topic(rg_name, resource_name, child_resource_name, oDataQuery, top)

        # Not a domain topic, invoke the standard list_by_resource
        return self.client.event_subscriptions.list_by_resource(
            rg_name,
            namespace,
            resource_type,
            resource_name,
            oDataQuery,
            top)


def main():
    AzureRMEventgridSubscriptionInfo()


if __name__ == '__main__':
    main()
